/*!******************************************************************/
/*  File: PM.h                                                      */
/*  ==========                                                      */
/*                                                                  */
/*  Common 'typedef' and '#define' declarations for PM's interface. */
/*                                                                  */
/*  This file includes the definition of the last part of the       */
/*  system global symbol table.                                     */
/*                                                                  */
/*  This file is part of the VARKON Pmpac Library.                  */
/*  URL:  http://www.varkon.com                                     */
/*                                                                  */
/*  This library is free software; you can redistribute it and/or   */
/*  modify it under the terms of the GNU Library General Public     */
/*  License as published by the Free Software Foundation; either    */
/*  version 2 of the License, or (at your option) any later         */
/*  version.                                                        */
/*                                                                  */
/*  This library is distributed in the hope that it will be         */
/*  useful, but WITHOUT ANY WARRANTY; without even the implied      */
/*  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR         */
/*  PURPOSE.  See the GNU Library General Public License for more   */
/*  details.                                                        */
/*                                                                  */
/*  You should have received a copy of the GNU Library General      */
/*  Public License along with this library; if not, write to the    */
/*  Free Software Foundation, Inc., 675 Mass Ave, Cambridge,        */
/*  MA 02139, USA.                                                  */
/*                                                                  */
/*  (C)Microform AB 1984-1999, J. Kjellander johan@microform.se     */
/*                                                                  */
/********************************************************************/
/* Names on structures and elements are constructed from the          */
/* following abbreviations.                                           */
/*              AT = ATTRIBUTE                                        */
/*              CO = CONDITION                                        */
/*              EX = EXPRESSION                                       */
/*              LI = LIST                                             */
/*              MD = MODIFIER                                         */
/*              MO = MODULE                                           */
/*              NO = NODE                                             */
/*              NP = NAMED PARAMETER                                  */
/*              PA = PARAMETER                                        */
/*              PM = Program Memory                                   */
/*              SB = SYMBOL TABLE                                     */
/*              ST = STATEMENT                                        */
/*              TY = TYPE                                             */

typedef char *string;   /* character pointer for names and string references */
                        /* the string ends with the NULL-character.          */

/* flyttat till common.h typedef DBint pm_ptr;    pointer type to PM structures, intern form nodes  */
                        /* and symbol table.                                 */ 
                        /* The definition of pm_ptr can be redefined under   */
                        /* the implementation face.                          */

typedef unsigned int pmseqn; /*PM sequence number in PM                      */

typedef short pmroutn;  /* Type for MBS routine identifier                   */


/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/*      pmint  -  definition       (INT_VAL)                                 */

typedef v2int pmint;     /* integer value type                               */

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/*      pmfloat  -  definition         (FLOAT_VAL)                           */

typedef v2float pmfloat; /* float value type                                 */

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

typedef v2vaty pmvaty;  /* Type for literal value structures with values     */
                        /* defined by constants.                             */
 
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
typedef char pmfieldt;  /* Type for VECTOR component fields.                 */

#define PM_X   1        /* constant for X-field                              */
#define PM_Y   2        /*    - " -     Y-field                              */
#define PM_Z   3        /*    - " -     Z-field                              */
	
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/

typedef char pmopty;    /* Type for binary operator.                         */

#define PM_GT     1     /* constant for  >      operator                     */
#define PM_GE     2     /*   - " -       >=      - " -                       */
#define PM_EQ     3     /*   - " -       =       - " -                       */ 
#define PM_NE     4     /*   - " -       <>      - " -                       */
#define PM_LT     5     /*   - " -       <       - " -                       */
#define PM_LE     6     /*   - " -       <=      - " -                       */
#define PM_NOT    7     /*   - " -      NOT      - " -                       */
#define PM_AND    8     /*   - " -      AND      - " -                       */
#define PM_OR     9     /*   - " -      OR       - " -                       */
#define PM_PLUS  10     /*   - " -       +       - " -                       */
#define PM_MINUS 11     /*   - " -       -       - " -                       */
#define PM_DIV   12     /*   - " -       /       - " -                       */
#define PM_MUL   13     /*   - " -       *       - " -                       */
#define PM_EXP   14     /*   - " -       **      - " -                       */
   
/*---------------------------------------------------------------------------*/
typedef char pmmoty;    /* type for Module type can have the value 2 (for 2D)*/
                        /* and  3 (for 3D). IGUNDEF = 0 => Not defined       */
#define _2D   2         /* value for 2D to be used with pmmoty               */
#define _3D   3         /* value for 3D        - " -                         */
/*---------------------------------------------------------------------------*/
typedef char pmmoat;    /* type for Module attribute IGUNDEF = Not defined   */
                        /*                                                   */
#define LOCAL  1        /* value for LOCAL to be used with pmmoat - type     */
#define GLOBAL 2        /* value for GLOBAL      - " -                       */
#define BASIC  3        /* value for BASIC       - " -                       */
#define MACRO  4        /* value for MACRO       -"-            92/02/13/JK  */
/*---------------------------------------------------------------------------*/
/* PMMODULE - interface structure for module header */

typedef struct pmmodule
    {
    char    mname[ V2SYNLEN ]; /* module name */
    pmmoty  mtype;      /* module type 2D eller 3D */
    pmmoat  mattri;     /* module attribute */
    pm_ptr  parlist;    /* parameter list */
    pm_ptr  stlist;     /* statement list */
    pmseqn  idmax;      /* maximal sequens number */
    DBint   ldsize;     /* local data area size */
    V3MDAT  system;     /* system data structure */
    } PMMODULE;

/*--------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/*      Literal value struct  -  definition                                  */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/*    The type for the following value structures are defined they represent */
/*    the different MBS literal types.                                       */
/*              pmint  -  INT_VAL      integer value  defined above          */
/*              pmfloat  -  FLOAT_VAL  float value       - " -               */
/*              string  -  STR_VAL     string value       - " -              */
/*              PMVECVA  -  VEC_VAL    vector value                          */
/*              PMREFVA  -  REF_VAL    reference value                       */
/*    It also contains value tages for address and boolean values            */
/*    internaly.                                                             */
/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/*      PMVECVA  -  definition           (VEC_VAL)                           */

typedef V2VECVA PMVECVA; /* vector value structure                           */

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/*      PMREFVA  -  definition            (REF_VAL)                          */

typedef V2REFVA PMREFVA; /* reference value structure                        */

/*- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -*/
/*     PMLITVA    Compond LITERAL struct - definition                        */

typedef V2LITVA PMLITVA; /* literal value structure                          */

/*---------------------------------------------------------------------------*/

/*---------------------------------------------------------------------------*/
/* PMPARVA - structure for parameter value and type */

typedef struct pmparva
    {
    PMLITVA  par_va;    /* parameter value */
    pm_ptr   par_ty;    /* parameter type */
    } PMPARVA;
/*--------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/
/* PMPATLOG - structure for log of module parameter type in GM */

typedef V2PATLOG PMPATLOG;

/*--------------------------------------------------------------------------*/
/*---------------------------------------------------------------------------*/
/* PMREFL - structure for reference-lists used in reference-analysis 920506JK*/

typedef struct pmrefl
  {
  DBseqnum snr;      /* Storhetens sekvensnummer */
  pm_ptr   statla ;  /* Satsens pm-adress */
  char     typ;      /* Typ av sats, Geo-Part-Proc-If etc. */
  } PMREFL;
/*--------------------------------------------------------------------------*/

/*
***Named parameters (attributes).
*/
typedef char pmmdid;  /* type for named parameters */

#define PMPEN    1    /* Constant for  PEN     */
#define PMLEVEL  2    /*    - " -      LEVEL   */
#define PMLFONT  3    /*    - " -      LFONT   */
#define PMAFONT  4    /*    - " -      AFONT   */
#define PMLDASHL 5    /*    - " -      LDASHL  */
#define PMADASHL 6    /*    - " -      ADASHL  */
#define PMTSIZE  7    /*    - " -      TSIZE   */
#define PMTWIDTH 8    /*    - " -      TWIDTH  */
#define PMTSLANT 9    /*    - " -      TSLANT  */
#define PMDTSIZE 10   /*    - " -      DTSIZE  */
#define PMDASIZE 11   /*    - " -      DASIZE  */
#define PMDNDIG  12   /*    - " -      DNDIG   */
#define PMDAUTO  13   /*    - " -      DAUTO   */
#define PMXFONT  14   /*    - " -      XFONT   */
#define PMXDASHL 15   /*    - " -      XDASHL  */
#define PMBLANK  16   /*    - " -      BLANK   */
#define PMHIT    17   /*    - " -      HIT     */
#define PMSAVE   18   /*    - " -      SAVE    */
#define PMCFONT  19   /*    - " -      CFONT   */
#define PMCDASHL 20   /*    - " -      CDASHL  */
#define PMTFONT  21   /*    - " -      TFONT   */
#define PMSFONT  22   /*    - " -      SFONT   */
#define PMSDASHL 23   /*    - " -      SDASHL  */
#define PMNULIN  24   /*    - " -      NULINES */
#define PMNVLIN  25   /*    - " -      NVLINES */
#define PMWIDTH  26   /*    - " -      WIDTH   */
#define PMTPMODE 27   /*    - " -      TPMODE  */

/*
***Definitions for PM internal structs.
*/
#include "../include/pmstru.h"
#include "../include/stdef.h"
#include "../include/ppdef.h"

/*
***Function prototypes for the PM API.
*/

/*
***eval1.c
*/
short evghdr(PMPARVA *pv);
short evgwdt(PMPARVA *pv, PMLITVA *valp);
short evpdat(PMPARVA *pv, PMLITVA *valp);
short evgdat(PMPARVA *pv, PMLITVA *valp);
short evddat(PMPARVA *pv, PMLITVA *valp);
short evgpoi(PMPARVA *pv);
short evglin(PMPARVA *pv);
short evgarc(PMPARVA *pv);
short evgcur(PMPARVA *pv);
short evgcuh(PMPARVA *pv);
short evgseg(PMPARVA *pv, short npar);
short evgsuh(PMPARVA *pv);
short evgtop(PMPARVA *pv);
short evgcub(PMPARVA *pv);
short evgfac(PMPARVA *pv);
short evgrat(PMPARVA *pv);
short evgtrf(PMPARVA *pv);
short evgtxt(PMPARVA *pv);
short evgxht(PMPARVA *pv);
short evgldm(PMPARVA *pv);
short evgcdm(PMPARVA *pv);
short evgrdm(PMPARVA *pv);
short evgadm(PMPARVA *pv);
short evggrp(PMPARVA *pv);
short evgcsy(PMPARVA *pv);
short evgbpl(PMPARVA *pv);
short evgprt(PMPARVA *pv);
short evuhdr(PMPARVA *pv);
short evtrim(PMPARVA *pv);
short evdel(PMPARVA *pv);
short evclgm(PMPARVA *pv);
short evblk(PMPARVA *pv);
short evublk(PMPARVA *pv);
short evwval(PMLITVA litval[], short nval, PMPARVA *pv);
short evwseg(short nseg, GMSEG seg[], short pn, PMPARVA *pv);
short evwfvk(gmflt fvk[], short nval, short pn, PMPARVA *pv);
short evwvec(DBVector vvk[], short nval, short pn, PMPARVA *pv);
short evwrve(PMREFVA idvek[], short i, short pn, PMPARVA *pv);

/*
***eval2.c
*/
short evldim(V2REFVA *id, PMPARVA pv[], V2NAPA *npblockp);
short evcdim(V2REFVA *id, PMPARVA pv[], V2NAPA *npblockp);
short evrdim(V2REFVA *id, PMPARVA pv[], V2NAPA *npblockp);
short evadim(V2REFVA *id, PMPARVA pv[], V2NAPA *npblockp);
short evxht(V2REFVA *id, PMPARVA  pv[], V2NAPA *npblockp, short npar);
short evarea(PMPARVA pv[], PMLITVA *valp, short npar);
short evsuar(PMPARVA pv[], PMLITVA *valp);
short evsear(PMPARVA pv[], PMLITVA *valp);
short evsecg(PMPARVA pv[], PMLITVA *valp);
short evcgra(PMPARVA pv[], PMLITVA *valp, short npar);
short evsymb(PMREFVA *id, PMPARVA *pv, V2NAPA *npblockp);
short evtext(PMREFVA *id, PMPARVA pv[], V2NAPA *npblockp);
short evbpln(V2REFVA *id, PMPARVA pv[], V2NAPA *npblockp);
short evtrmv(V2REFVA *id, PMPARVA pv[], V2NAPA *npblockp, short np);
short evtrrp(V2REFVA *id, PMPARVA pv[], V2NAPA *npblockp, short np);
short evtrrl(V2REFVA *id, PMPARVA pv[], V2NAPA *npblockp, short np);
short evtrmi(V2REFVA *id, PMPARVA pv[], V2NAPA *npblockp, short np);
short evtrco(V2REFVA *id, PMPARVA pv[], V2NAPA *npblockp);
short evtrus(V2REFVA *id, PMPARVA pv[], V2NAPA *npblockp);
short evtcop(V2REFVA *id, PMPARVA pv[], V2NAPA *npblockp);

/*
***eval3.c
*/
short evsrgm(PMPARVA *pv, short np);
short evgngm(PMPARVA *pv, PMLITVA *valp);
short evgtid(PMPARVA *pv, PMLITVA *valp);
short evgtyp(PMPARVA *pv, PMLITVA *valp);
short evgint(PMPARVA *pv, PMLITVA *valp);
short evgflt(PMPARVA *pv, PMLITVA *valp);
short evgstr(PMPARVA *pv, PMLITVA *valp);
short evgvec(PMPARVA *pv, PMLITVA *valp);
short evgref(PMPARVA *pv, PMLITVA *valp);
short evrstr(PMPARVA *pv, PMLITVA *valp);
short evrval(PMPARVA *pv, PMLITVA *valp);
short evival(PMPARVA *pv, PMLITVA *valp, short np);
short evfval(PMPARVA *pv, PMLITVA *valp, short np);
short evrefc(PMPARVA *pv, PMLITVA *valp, short np);
short evglor(PMPARVA *pv, PMLITVA *valp);
short evfins(PMPARVA *pv, PMLITVA *valp);
short evvecl(PMPARVA *pv, PMLITVA *valp);
short evvecn(PMPARVA *pv, PMLITVA *valp);
short evcrpr(PMPARVA *pv, PMLITVA *valp);
short evangl(PMPARVA *pv, PMLITVA *valp);
short evgtts(PMPARVA *pv, PMLITVA *valp);
short evgtal(PMPARVA *pv, PMLITVA *valp);
short evinpt(PMPARVA *pv, PMLITVA *valp);
short evinfn(PMPARVA *pv, PMLITVA *valp);

/*
***eval4.c
*/
short evldgm(PMPARVA *pv);
short evsvgm(PMPARVA *pv);
short evldjb(PMPARVA *pv, short npar);
short evsvjb(PMPARVA *pv);
short evbllv(PMPARVA *pv);
short evublv(PMPARVA *pv);
short evgtlv(PMPARVA *pv);
short evgtvi(PMPARVA *pv);
short evcrvi(PMPARVA *pv);
short evacvi(PMPARVA *pv);
short evscvi(PMPARVA *pv);
short evcevi(PMPARVA *pv);
short evhdvi(PMPARVA *pv, short npar);
short evprvi(PMPARVA *pv);
short evervi(PMPARVA *pv);
short evrpvi(PMPARVA *pv, short npar);
short evcavi(PMPARVA *pv);
short evplvi(PMPARVA *pv);
short evplwi(PMPARVA *pv, short np);
short evmsiz(PMPARVA *pv);
short evgrid(PMPARVA *pv);
short evgrix(PMPARVA *pv);
short evgriy(PMPARVA *pv);
short evgrdx(PMPARVA *pv);
short evgrdy(PMPARVA *pv);
short evtime(PMPARVA *pv);
short evlsin(PMPARVA *pv);
short evlsex(PMPARVA *pv);
short evlsln(PMPARVA *pv);
short evpspt(PMPARVA *pv);
short evpopt(PMPARVA *pv);
short evcrts(PMPARVA *pv);
short evcrmy(PMPARVA *pv, short npar);
short evpsmy(PMPARVA *pv);
short evpomy(PMPARVA *pv);
short evgtmy(PMPARVA *pv);
short evlmdf(PMPARVA *pv);
short evexst(PMLITVA *valp);

/*
***eval5.c
*/
short evalev(PMLITVA *valp);
short evablk(PMLITVA *valp);
short evavin(PMLITVA *valp);
short evapen(PMLITVA *valp);
short evawdt(PMLITVA *valp);
short evascl(PMLITVA *valp);
short evadsc(PMLITVA *valp);
short evacac(PMLITVA *valp);
short evagrx(PMLITVA *valp);
short evagry(PMLITVA *valp);
short evagdx(PMLITVA *valp);
short evagdy(PMLITVA *valp);
short evagrd(PMLITVA *valp);
short evajbn(PMLITVA *valp);
short evamtp(PMLITVA *valp);
short evamat(PMLITVA *valp);
short evajbd(PMLITVA *valp);
short evalft(PMLITVA *valp);
short evaaft(PMLITVA *valp);
short evacft(PMLITVA *valp);
short evaxft(PMLITVA *valp);
short evaldl(PMLITVA *valp);
short evaadl(PMLITVA *valp);
short evacdl(PMLITVA *valp);
short evaxdl(PMLITVA *valp);
short evatsi(PMLITVA *valp);
short evatwi(PMLITVA *valp);
short evatsl(PMLITVA *valp);
short evadts(PMLITVA *valp);
short evadas(PMLITVA *valp);
short evadnd(PMLITVA *valp);
short evadau(PMLITVA *valp);
short evacsy(PMLITVA *valp);
short evapid(PMLITVA *valp);
short evavvr(PMLITVA *valp);
short evavsr(PMLITVA *valp);
short evaost(PMLITVA *valp);
short evahst(PMLITVA *valp);
short evcrnp(PMLITVA *valp);
short evcrmp(PMLITVA *valp);

/*
***eval6.c
*/
short evpofr(V2REFVA *id, PMPARVA pv[], pmroutn rout, V2NAPA  *npb);
short evlifr(V2REFVA *id, PMPARVA pv[], pmroutn rout, V2NAPA  *npb);
short evliof(V2REFVA *id, PMPARVA pv[], V2NAPA *npb);
short evlian(V2REFVA *id, PMPARVA pv[], V2NAPA *npb);
short evlit1(V2REFVA *id, PMPARVA pv[], V2NAPA *npb);
short evlit2(V2REFVA *id, PMPARVA pv[], V2NAPA *npb);
short evlipe(V2REFVA *id, PMPARVA pv[], V2NAPA *npb);
short evarfr(V2REFVA *id, PMPARVA pv[], V2NAPA *npb);
short evar2p(V2REFVA *id, PMPARVA pv[], V2NAPA *npb);
short evar3p(V2REFVA *id, PMPARVA pv[], V2NAPA *npb);
short evarof(V2REFVA *id, PMPARVA pv[], V2NAPA *npb);
short evarfl(V2REFVA *id, PMPARVA pv[], V2NAPA *npb);
short evcufr(V2REFVA *id, PMPARVA pv[], pmroutn rout, V2NAPA *npb, short npar);
short evcurs(V2REFVA *id, PMPARVA pv[], V2NAPA *npb, short np);
short evcusa(V2REFVA *id, PMPARVA pv[], V2NAPA *npb, short np);
short evcurc(V2REFVA *id, PMPARVA pv[], V2NAPA *npb, short np);
short evcuca(V2REFVA *id, PMPARVA pv[], V2NAPA *npb);
short evcuro(V2REFVA *id, PMPARVA pv[], V2NAPA *npb);
short evcomp(V2REFVA *id, PMPARVA pv[], V2NAPA *npb, short np);
short evcurt(V2REFVA *id, PMPARVA pv[], V2NAPA *npb);
short evcuru(V2REFVA *id, PMPARVA pv[], V2NAPA *npb, short np);
short evcs3p(V2REFVA *id, PMPARVA pv[], V2NAPA *npb, short np);
short evcs1p(V2REFVA *id, PMPARVA pv[], V2NAPA *npb, short np);
short evgrp(V2REFVA *id, PMPARVA pv[], V2NAPA *npb, short np);

/*
***eval7.c
*/
short evdbop(PMPARVA *pv, PMLITVA *valp);
short evdbbg(PMPARVA *pv, PMLITVA *valp);
short evdbsl(PMPARVA *pv, PMLITVA *valp, short npar);
short evdbup(PMPARVA *pv, PMLITVA *valp, short npar);
short evdbin(PMPARVA *pv, PMLITVA *valp, short npar);
short evdbdl(PMPARVA *pv, PMLITVA *valp);
short evdben(PMLITVA *valp);
short evdbrl(PMLITVA *valp);
short evdbcl(PMLITVA *valp);
short evdbfi(PMPARVA *pv, PMLITVA *valp);
short evdbff(PMPARVA *pv, PMLITVA *valp);
short evdbfs(PMPARVA *pv, PMLITVA *valp);
short evdbnx(PMLITVA *valp);

/*
***eval8.c
*/
short evaddm(PMPARVA *pv, PMLITVA *valp, short npar);
short evdelm(PMPARVA *pv);
short evrunm(PMPARVA *pv);
short evgpm(PMPARVA *pv, PMLITVA *valp);
short evupm(PMPARVA *pv);
short evuppm(PMPARVA *pv);
short evclpm();
short evposm(PMPARVA *pv, PMLITVA *valp);
short evldpm(PMPARVA *pv);

/*
***eval9.c
*/
short evsexd(V2REFVA *ident, PMPARVA pv[], V2NAPA *npblockp);
short evscon(V2REFVA *ident, PMPARVA pv[], V2NAPA *npblockp, short np);
short evsoff(V2REFVA *ident, PMPARVA pv[], V2NAPA *npblockp);
short evssar(V2REFVA *ident, PMPARVA pv[], V2NAPA *npblockp, short np);
short evscar(V2REFVA *ident, PMPARVA pv[], V2NAPA *npblockp);
short evsnar(V2REFVA *ident, PMPARVA pv[], V2NAPA *npblockp, short np);
short evsapp(V2REFVA *ident, PMPARVA pv[], V2NAPA *npblockp, short np);
short evscom(V2REFVA *ident, PMPARVA pv[], V2NAPA *npblockp, short np);
short evstrm(V2REFVA *ident, PMPARVA pv[], V2NAPA *npblockp, short np);
short evsrot(V2REFVA *ident, PMPARVA pv[], V2NAPA *npblockp);
short evsusr(V2REFVA *ident, PMPARVA pv[], V2NAPA *npblockp);
short evsswp(V2REFVA *ident, PMPARVA pv[], V2NAPA *npblockp);
short evscyl(V2REFVA *ident, PMPARVA pv[], V2NAPA *npblockp);
short evsrul(V2REFVA *ident, PMPARVA pv[], V2NAPA *npblockp);
short evscur(V2REFVA *ident, PMPARVA pv[], V2NAPA *npblockp);
short evcsil(V2REFVA *ident, PMPARVA pv[], V2NAPA *npblockp);
short evcint(V2REFVA *ident, PMPARVA pv[], V2NAPA *npblockp);
short evciso(V2REFVA *ident, PMPARVA pv[], V2NAPA *npblockp);
short evcfan(V2REFVA *ident, PMPARVA pv[], V2NAPA *npblockp, short np);
short evcged(V2REFVA *ident, PMPARVA pv[], V2NAPA *npblockp);
short evcapp(V2REFVA *ident, PMPARVA pv[], V2NAPA *npblockp, short np);
short evcugl(V2REFVA *ident, PMPARVA pv[], V2NAPA *npblockp);
short evions(PMPARVA pv[], PMLITVA *valp, short np);
short eveval(PMPARVA pv[], PMLITVA *valp);
short evncui(PMPARVA pv[], PMLITVA *valp);
short evails(PMPARVA pv[], short np);
short evailm(PMPARVA pv[], short np);
short evaios(PMPARVA pv[], short np);

/*
***eval10.c
*/
short evcrwi(PMPARVA pv[], PMLITVA *valp);
short evcred(PMPARVA pv[], PMLITVA *valp);
short evcrbu(PMPARVA pv[], PMLITVA *valp, short npar);
short evcrdb(PMPARVA pv[], PMLITVA *valp, short npar);
short evcrfb(PMPARVA pv[], PMLITVA *valp);
short evcric(PMPARVA pv[], PMLITVA *valp, short npar);
short evcrfi(PMPARVA pv[], PMLITVA *valp);
short evgted(PMPARVA pv[], PMLITVA *valp);
short evgtbu(PMPARVA pv[], PMLITVA *valp);
short evgtwi(PMPARVA pv[], short np);
short evshwi(PMPARVA pv[]);
short evwtwi(PMPARVA pv[], PMLITVA *valp);
short evdlwi(PMPARVA pv[], short np);
short evgttl(PMPARVA pv[], PMLITVA *valp, short np);
short evgtth(PMPARVA pv[], PMLITVA *valp, short np);

/*
***eval11.c
*/
short evappf(PMPARVA pv[]);
short evdelf(PMPARVA pv[]);
short evmovf(PMPARVA pv[]);
short evcopf(PMPARVA pv[]);
short evtstf(PMPARVA pv[], PMLITVA *valp);
short evgenv(PMPARVA pv[], PMLITVA *valp);

/*
***eval12.c
*/
short evlddl(PMPARVA pv[], PMLITVA *valp);
short evcldl(PMPARVA pv[], short np);
short evuldl(PMPARVA pv[]);

/*
***eval13.c
*/
short evopen(PMPARVA pv[]);
short evclos(PMPARVA pv[], short np);
short evoutl(PMPARVA pv[]);
bool  v3isop(char *path);

/*
***eval14.c
*/
short evltvi(PMPARVA pv[], short np);
short evlton(PMPARVA pv[]);
short evltof(PMPARVA pv[]);
short evshvi(PMPARVA pv[]);
short evmtvi(PMPARVA pv[]);
short evcrco(PMPARVA pv[]);

/*
***eval15.c
*/
short evgptr(PMPARVA pv[], PMLITVA *valp);
short evscr(PMPARVA pv[], PMLITVA *valp, short np);
short evscr2(PMPARVA pv[]);
short evpos(PMPARVA *pv, PMLITVA *valp, short np);
short evpos2(PMPARVA pv[]);
short evidnt(PMPARVA pv[], PMLITVA *valp, short np);
short evidn2(PMPARVA pv[]);
short evhigh(PMPARVA pv[], short np);

/*
***eval16.c
*/
short evodco(PMPARVA pv[], PMLITVA *valp);
short evoded(PMPARVA pv[], PMLITVA *valp);
short evodfe(PMLITVA *valp);
short evodgs(PMPARVA pv[], PMLITVA *valp);
short evoddc(PMLITVA *valp);
short evoder(PMLITVA *valp);

/*
***inac10.c
*/
short inevex(pm_ptr exprla, PMLITVA *valp, pm_ptr *tylap);
short inevev(pm_ptr exprla, PMLITVA *valp, pm_ptr *tylap);
short evabs(pm_ptr exlist, PMLITVA *valp, pm_ptr *tylap);

/*
***inac11.c
*/
#ifndef ANALYZER
short evrand(PMPARVA pv[], PMLITVA *valp, short npar);
short evarcl(PMPARVA pv[], PMLITVA *valp); 
short eviarc(PMPARVA pv[], PMLITVA *valp);
short evtxtl(PMPARVA pv[], PMLITVA *valp);
short evtang(PMPARVA pv[], PMLITVA *valp);
short evitan(PMPARVA pv[], PMLITVA *valp);
short evcurv(PMPARVA pv[], PMLITVA *valp);
short evicur(PMPARVA pv[], PMLITVA *valp);
short evcen(PMPARVA pv[], PMLITVA *valp);
short evnorm(PMPARVA pv[], PMLITVA *valp, short npar);
short evstrt(PMPARVA pv[], PMLITVA *valp);
short evend(PMPARVA pv[], PMLITVA *valp);
short evon(PMPARVA pv[], PMLITVA *valp, short npar);
short evion(PMPARVA pv[], PMLITVA *valp);
short evsect(PMPARVA pv[], PMLITVA *valp);
short evnsec(PMPARVA pv[], PMLITVA *valp);
#endif
short evvec(PMPARVA pv[], PMLITVA *valp);
short evvecp(PMPARVA pv[], PMLITVA *valp);

/*
***inac12.c
*/
short evpart(V2REFVA *ident, PMPAST *partp);

/*
***inac13.c
*/
short inevnl(pm_ptr nplist, V2NAPA *npblockp);
short insetb(pm_ptr nplist);
short inpunp(V2NAPA *newdefnp);
short inponp();
short incdnp(V2NAPA *nppek, short kod);

/*
***inac14.c
*/
short evmolo(PMPARVA pv[]);
short evmogl();
short evmoba();
short evset(pm_ptr exlist);
short evsetb(pm_ptr exlist);
short evos(PMPARVA pv[], short npar);
short evexit(PMPARVA pv[], short npar);

/*
***inac17.c
*/
short evequa(PMLITVA *val1p, pm_ptr tyla1, PMLITVA *val2p, pm_ptr tyla2,
             PMLITVA *valp, pm_ptr *tylap);
short evgreq(PMLITVA *val1p, pm_ptr tyla1, PMLITVA *val2p, pm_ptr tyla2,
             PMLITVA *valp, pm_ptr *tylap);
short evleeq(PMLITVA *val1p, pm_ptr tyla1, PMLITVA *val2p, pm_ptr tyla2,
             PMLITVA *valp, pm_ptr *tylap);
short evand(pm_ptr rexpr, PMLITVA *val1p, pm_ptr tyla1,
            PMLITVA *valp, pm_ptr *tylap);
short evor(pm_ptr rexpr, PMLITVA *val1p, pm_ptr tyla1,
           PMLITVA *valp, pm_ptr *tylap);

/*
***inac18.c
*/
short evseek(PMPARVA pv[]);
short evouti(PMPARVA pv[]);
short evoutf(PMPARVA pv[]);
short evouts(PMPARVA pv[]);
short evoutb(PMPARVA pv[]);
short evinio(pmvaty iotyp, PMPARVA pv[], PMLITVA *valp); 
short evinl(PMPARVA pv[], PMLITVA *valp);
short evinb(PMPARVA pv[], short np);
short evfpos(PMPARVA pv[], PMLITVA *valp);
short eviost(PMPARVA pv[], PMLITVA *valp);
short evufnm(PMLITVA *valp);

/*
***inac19.c
*/
short evchr(PMPARVA pv[], PMLITVA *valp);
short evasci(PMPARVA pv[], PMLITVA *valp);
short evstr(PMPARVA pv[], PMLITVA *valp);
short evval(PMPARVA pv[], PMLITVA *valp);
short evleng(PMPARVA pv[], PMLITVA *valp);
short evsubs(PMPARVA pv[], PMLITVA *valp);
short mkform(pmvaty typ, int form1, int form2, char *formstr);

/*
***inac20.c
*/
short inwvar(pm_ptr acttyla, DBint rtsaddr, short dim,
             short indarr[], PMLITVA *valp);
short ingpva(pm_ptr palist, char valarr[], PMPATLOG typarr[], int *par_nr);

/*
***inacc.c
*/
short inmod(pm_ptr mola);
short inssta(pm_ptr statla);
short ininit();
short inevst(pm_ptr statla);
short inevsl(pm_ptr stlist);
short inevel(pm_ptr exlist);
short inevpa(pm_ptr pala, PMLITVA *valp, STTYTBL *in_tp, DBint cfsize);
short inevpl(pm_ptr palist);
short inevmo(pm_ptr mola);
short inevcl(pm_ptr colist);
short inevco(pm_ptr condla);
short inev(pm_ptr a);
short ineqty(pm_ptr tyla1, pm_ptr tyla2);
short incoty(PMLITVA *avalp, pm_ptr *atylap, pm_ptr ctyla);

/*
***infunc.c
*/
short inevfu(pm_ptr routla, pm_ptr arglist, PMLITVA *valp, pm_ptr *tylap);

/*
***ingeop.c
*/
short inevgp(V2REFVA *identp, pm_ptr routla, pm_ptr arglist, V2NAPA *npblockp);

/*
***ingpar.c
*/
short inapar(pm_ptr actplist, pm_ptr routla, STPROC *rout,
             PMPARVA **pv, short *nopar);
void  inrpar();
void  inclpp();

/*
***innpar.c
*/
short inrdnp();
short inevnp(pm_ptr napala, V2NAPA *npblockp);

/*
***inproc.c
*/
short inevpr(pm_ptr routla, pm_ptr arglist);

/*
***inrts.c
*/
short inpfrh(char *mnamp, DBint datasz);
short ingfrh(char *mnamp);
short inpdfr(DBint datasize);
short inpval(DBint rtsofs, pm_ptr tyla, bool addrflg, PMLITVA *litvap);
short ingval(DBint rtsofs, pm_ptr tyla, bool addrflg, PMLITVA *litvap);
void  ingref(DBint rtsofs, PMLITVA *litvap);
short inirts();
short incrts();
short inallo(DBint size, char **chp, DBint *rtsofs);
char *incrsa(DBint rtsofs);
short insrtb(char *rbasp);
short insrsp(char *rsp);
char *ingrtb();
char *inggrb();
char *inglrb();
char *ingrsp();
short inslrb();

/*
***pmac1.c
*/
short pminit();
short pmcmod(char *name, PMMODULE *module, pm_ptr *retla);
short pmload(char  *modname, pm_ptr *retla);
short pmsave(pm_ptr mola);
short pmlmst(pm_ptr modptr, pm_ptr statptr);
short pmlmpa(pm_ptr modptr, pm_ptr parptr);
short pmumod(PMMODULE *module);
short pmrmod(PMMODULE *module);
short pmcpar(pm_ptr symbp, char *prompt, pm_ptr *retla);
short pmcass(pm_ptr lvalue, pm_ptr rvalue, pm_ptr *retla);
short pmcifs(pm_ptr condl, pm_ptr statl, pm_ptr *retla);
short pmcfos(pm_ptr loopvar, pm_ptr startex, pm_ptr toex,
             pm_ptr stepex, pm_ptr statl, pm_ptr *retla);
short pmcgos(pm_ptr label, pm_ptr *retla);
short pmclas(pm_ptr label, pm_ptr stat, pm_ptr *retla);
short pmcprs(pm_ptr nameid, pm_ptr valparam, pm_ptr *retla);
short pmcpas(pmseqn geid, char  *mname, pm_ptr mnexpr, pm_ptr mpara,
             pm_ptr ppara, pm_ptr nampara, pm_ptr *retla);
short pmcges(pm_ptr nameid, pmseqn geid, pm_ptr valparam,
             pm_ptr namparam, pm_ptr *retla);
short pmcune(pmopty opid, pm_ptr expr, pm_ptr *retla);
short pmcbie(pmopty opid, pm_ptr lexpr, pm_ptr rexpr, pm_ptr *retla);
short pmclie(PMLITVA *lit_p, pm_ptr *retla);
short pmcide(pm_ptr id_p, pm_ptr *retla);
short pmcine(pm_ptr indid_p, pm_ptr experl, pm_ptr *retla);
short pmccoe(pm_ptr comvar, pmfieldt field, pm_ptr *retla);
short pmcfue(pm_ptr fid, pm_ptr arglist, pm_ptr *retla);
short pmcnpa(pmmdid paname, pm_ptr valuep, pm_ptr *retla);
short pmccon(pm_ptr cond, pm_ptr statl, pm_ptr *retla);
short pmtcon(pm_ptr gennop, pm_ptr listp, pm_ptr *retla, pm_ptr *lnodep);
short pmcstr(char *str, pm_ptr *retla);
short pmcref(PMREFVA *refp, pm_ptr *retla);
void  pmstpr(pm_ptr mbase);

/*
***pmac2.c
*/
short pmgmod(pm_ptr pmla, PMMONO **retp);
short pmgpar(pm_ptr pmla, PMPANO **retp);
short pmgsta(pm_ptr pmla, PMSTNO **retp);
short pmgexp(pm_ptr pmla, PMEXNO **retp);
short pmgnpa(pm_ptr pmla, PMNPNO **retp);
short pmgcon(pm_ptr pmla, PMCONO **retp);
short pmglin(pm_ptr pmla, pm_ptr *nextla, pm_ptr *genola);
short pmgfli(pm_ptr pmla, pm_ptr *retla);
short pmgstr(pm_ptr pmla, char **retp);
short pmgvec(pm_ptr pmla, PMVECVA **retp);
short pmgref(pm_ptr pmla, PMREFEX **retp);

/*
***pmac3.c
*/
bool pmargs(pm_ptr statla);
bool pmarex(PMREFVA *idvek, pm_ptr exprla);
bool pmamir(PMREFVA *id);

/*
***pmac4.c
*/
short pmrpap(pm_ptr modptr);
short pmgpad(pm_ptr *panola);
short pmrpar(pm_ptr panola, char *name, char *prompt, PMLITVA *litval);
short pmupar(pm_ptr panola, PMLITVA *litval);

/*
***pmac5.c
*/
short pmdges(V2REFVA *ref);
short pmdlst();
short pmchpa(V2REFVA *ref, short nr, pm_ptr new_expr, pm_ptr *retla);
short pmchnp(V2REFVA *ref, pmmdid npid, pm_ptr new_expr, pm_ptr *retla);
short pmlges(V2REFVA *ref, pm_ptr  *stllap, pm_ptr  *retla);
short pmrgps(pm_ptr  lstla, pm_ptr  newsla);
short pmgpla(V2REFVA *id, short *pant, pm_ptr pla[]);

/*
***pmac6.c
*/
short pmwrme(PMREFVA *id, PMREFL **prtabp, int *rant);

/*
***pmallo.c
*/
short  pmibuf();
short  pmallo(DBint blsize, pm_ptr *retla);
short  pmgeba(char *moname, pm_ptr *retla);
short  pmclea();
short  pmrset();
char  *pmgadr(pm_ptr pmla);
short  pmsbla(pm_ptr basla);
pm_ptr pmgbla();
short  pmsaba(pm_ptr basla);
pm_ptr pmgaba();
short  pmssta();
short  pmlsta(char *moname, pm_ptr *retla);
void   pmmark();
void   pmrele();
void   pmgstp(pm_ptr *pstack);
void   pmsstp(pm_ptr stkptr);
void   pmstat(short  *pnm, char namvek[][JNLGTH+1], pm_ptr basvek[],
             pm_ptr sizvek[], pm_ptr *pstack, pm_ptr *pheap);
short  clheap();

/*
***pmlist.c
*/
short pmprmo(pm_ptr mola);

/*
**pretty.c
*/
short pprsts(pm_ptr statla, short geotyp, char *str, int ntkn);
short pprst(pm_ptr statla);
short pprexs(pm_ptr exprla, short geotyp, char *str, int ntkn);
short pprex(pm_ptr exprla, ppopri inpri);
short pprmo(char mode, FILE *filpek);

/*
***calext.c
*/
short evcaxt(PMPARVA pv[]);

