/**********************************************************************
*
*    eval14.c
*    ========
*
*    This file is part of the VARKON Program Module Library.
*    URL: http://www.varkon.com
*
*    This file includes the following routines:
*
*    evltvi();     Evaluerar LIGHT_VIEW
*    evlton();     Evaluerar LIGHT_ON
*    evltof();     Evaluerar LIGHT_OFF
*    evshvi();     Evaluerar SHADE_VIEW
*    evmtvi();     Evaluerar MATERIAL_VIEW
*    evcrco();     Evaluerar CRE_COLOR
*
*    This library is free software; you can redistribute it and/or
*    modify it under the terms of the GNU Library General Public
*    License as published by the Free Software Foundation; either
*    version 2 of the License, or (at your option) any later version.
*
*    This library is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*    Library General Public License for more details.
*
*    You should have received a copy of the GNU Library General Public
*    License along with this library; if not, write to the Free
*    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
*    (C)Microform AB 1984-1999, Johan Kjellander, johan@microform.se
*
***********************************************************************/

#include "../../DB/include/DB.h"
#include "../../IG/include/IG.h"
#ifdef V3_X11
#include "../../WP/include/WP.h"
#endif

/*!******************************************************/

        short evltvi(
        PMPARVA  pv[],
        short    np)

/*      Evaluerar LIGHT_VIEW.
 *
 *      In: pv   => Pekare till array med parametervrden
 *          np   => Antal parametrar.
 *
 *      Felkod: 
 *
 *      (C)microform ab 1997-02-18 J. Kjellander
 *
 ******************************************************!*/

 {
   DBVector dir;
   gmflt spotang,focus;


   if ( np == 2 )
     {
     dir.x_gm = 0.0;
     dir.y_gm = 0.0;
     dir.z_gm = 1.0;
     spotang  = 180.0;
     focus    = 0.0;
     }
   else if ( np == 3 )
     {
     dir.x_gm = pv[3].par_va.lit.vec_va.x_val;
     dir.y_gm = pv[3].par_va.lit.vec_va.y_val;
     dir.z_gm = pv[3].par_va.lit.vec_va.z_val;
     spotang  = 180.0;
     focus    = 0.0;
     }
   else if ( np == 4 )
     {
     dir.x_gm = pv[3].par_va.lit.vec_va.x_val;
     dir.y_gm = pv[3].par_va.lit.vec_va.y_val;
     dir.z_gm = pv[3].par_va.lit.vec_va.z_val;
     spotang  = pv[4].par_va.lit.float_va;
     focus    = 0.0;
     }
   else
     {
     dir.x_gm = pv[3].par_va.lit.vec_va.x_val;
     dir.y_gm = pv[3].par_va.lit.vec_va.y_val;
     dir.z_gm = pv[3].par_va.lit.vec_va.z_val;
     spotang  = pv[4].par_va.lit.float_va;
     focus    = pv[5].par_va.lit.float_va;
     }

   return(wpltvi(pv[1].par_va.lit.int_va,
    (DBVector *)&pv[2].par_va.lit.vec_va,
                &dir,spotang,focus));
 }

/********************************************************/
/*!******************************************************/

        short evlton(PMPARVA pv[])

/*      Evaluerar LIGHT_ON.
 *
 *      In: pv   => Pekare till array med parametervrden
 *
 *      (C)microform ab 1997-02-19 J. Kjellander
 *
 ******************************************************!*/

 {
   return(wplton(pv[1].par_va.lit.int_va,
                 pv[2].par_va.lit.float_va,
                 TRUE));
 }

/********************************************************/
/*!******************************************************/

        short evltof(PMPARVA pv[])

/*      Evaluerar LIGHT_OFF.
 *
 *      In: pv   => Pekare till array med parametervrden
 *
 *      (C)microform ab 1997-02-19 J. Kjellander
 *
 ******************************************************!*/

 {
   return(wplton(pv[1].par_va.lit.int_va,
                 100.0,
                 FALSE));
 }

/********************************************************/
/*!******************************************************/

        short evshvi(PMPARVA pv[])

/*      Evaluerar SHADE_VIEW.
 *
 *      In: pv   => Pekare till array med parametervrden
 *
 *      Felkod: 
 *
 *      (C)microform ab 1997-02-18 J. Kjellander
 *
 ******************************************************!*/

 {
   return(wpshad(pv[1].par_va.lit.int_va,
                 pv[2].par_va.lit.int_va));
 }

/********************************************************/
/*!******************************************************/

        short evmtvi(PMPARVA pv[])

/*      Evaluerar MATERIAL_VIEW.
 *
 *      In: pv   => Pekare till array med parametervrden
 *
 *      Felkod: 
 *
 *      (C)microform ab 1997-02-18 J. Kjellander
 *
 ******************************************************!*/

 {
   return(wpmtvi(pv[1].par_va.lit.int_va,
                 pv[2].par_va.lit.float_va,
                 pv[3].par_va.lit.float_va,
                 pv[4].par_va.lit.float_va,
                 pv[5].par_va.lit.float_va,
                 pv[6].par_va.lit.float_va,
                 pv[7].par_va.lit.float_va,
                 pv[8].par_va.lit.float_va,
                 pv[9].par_va.lit.float_va,
                 pv[10].par_va.lit.float_va,
                 pv[11].par_va.lit.float_va,
                 pv[12].par_va.lit.float_va,
                 pv[13].par_va.lit.float_va,
                 pv[14].par_va.lit.float_va));
 }

/********************************************************/
/*!******************************************************/

        short evcrco(PMPARVA pv[])

/*      Evaluerar CRE_COLOR
 *
 *      In: pv   => Pekare till array med parametervrden
 *
 *      Felkod: 
 *
 *      (C)microform ab 1997-02-18 J. Kjellander
 *
 ******************************************************!*/

 {
#ifdef V3_X11

   return(wpccol(pv[1].par_va.lit.int_va,
                 pv[2].par_va.lit.int_va,
                 pv[3].par_va.lit.int_va,
                 pv[4].par_va.lit.int_va));
#endif

#ifdef WIN32
extern int msccol();

   return(msccol(pv[1].par_va.lit.int_va,
                 pv[2].par_va.lit.int_va,
                 pv[3].par_va.lit.int_va,
                 pv[4].par_va.lit.int_va));
#endif

 }

/********************************************************/
