/**********************************************************************
*
*    eval15.c
*    ========
*
*    This file is part of the VARKON Program Module Library.
*    URL: http://www.varkon.com
*
*    This file includes the following routines:
*
*    evgptr();    Evaluerar GET_POINTER
*    evscr();     Evaluerar SCREEN
*    evscr2();    Evaluerar SCREEN_2
*    evpos();     Evaluerar POS
*    evpos2();    Evaluerar POS_2
*    evidnt();    Evaluerar IDENT
*    evidn2();    Evaluerar IDENT_2
*    evhigh();    Evaluerar HIGHLIGHT
*
*    This library is free software; you can redistribute it and/or
*    modify it under the terms of the GNU Library General Public
*    License as published by the Free Software Foundation; either
*    version 2 of the License, or (at your option) any later version.
*
*    This library is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*    Library General Public License for more details.
*
*    You should have received a copy of the GNU Library General Public
*    License along with this library; if not, write to the Free
*    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
*    (C)Microform AB 1984-1999, Johan Kjellander, johan@microform.se
*
***********************************************************************/

#include "../../DB/include/DB.h"
#include "../../IG/include/IG.h"
#include "../../EX/include/EX.h"
#include "../../GP/include/GP.h"

#ifdef V3_X11
#include <X11/Xlib.h>
#include "../../WP/include/WP.h"
#endif

#ifdef WIN32
#include "../../WP/include/WP.h"
#endif

#ifdef WIN32
extern int msgtsw(),msgtmp();
#endif

/*!******************************************************/

        short evgptr(
        PMPARVA  pv[],
        PMLITVA *valp)

/*      Evaluerar funktionen GET_POINTER.
 *
 *      In: valp => Pekare till funktionsvrde.
 *
 *      (C)microform ab 1998-03-23 J. Kjellander
 *
 *      1999-05-05 New param, J.Kjellander
 *
 ******************************************************!*/

 {
   int     intx,inty,main_x,main_y,gra_x,gra_y;
   WPGWIN *gwinpt;
/*
***Get current mouse position in screen coordinates.
*/
#ifdef V3_X11

   wpgtmp(&intx,&inty);
#endif
#ifdef WIN32
   int dum1,dum2;
   msgtmp(&intx,&inty);
#endif

/*
***Adjust according to code.
*/
   switch ( pv[1].par_va.lit.int_va )
     {
/*
***Relative to screen.
*/
     case 0:
     valp->lit.vec_va.x_val = (DBfloat)intx;
     valp->lit.vec_va.y_val = (DBfloat)inty;
     valp->lit.vec_va.z_val = 0.0;
     return(0);
/*
***Relative to application main window ?
*/
     case 1:
#ifdef V3_X11
     gwinpt = (WPGWIN *)wpwtab[GWIN_MAIN].ptr;
     wpgtwp(gwinpt->id.x_id,&main_x,&main_y);
#endif
#ifdef WIN32
     msggeo(ms_main,&main_x,&main_y,&dum1,&dum2,NULL,NULL);
#endif
     valp->lit.vec_va.x_val = (DBfloat)(intx - main_x);
     valp->lit.vec_va.y_val = (DBfloat)(inty - main_y);
     valp->lit.vec_va.z_val = 0.0;
     return(0);
/*
***Relative to graphics window 0 ?
*/
     case 2:
     gwinpt = (WPGWIN *)wpwtab[GWIN_MAIN].ptr;
#ifdef V3_X11
     wpgtwp(gwinpt->id.x_id,&gra_x,&gra_y);
     valp->lit.vec_va.x_val = (DBfloat)(intx - gra_x);
     valp->lit.vec_va.y_val = (DBfloat)(inty - gra_y);
     valp->lit.vec_va.z_val = 0.0;
#endif
#ifdef WIN32
     msggeo(ms_main,&main_x,&main_y,&dum1,&dum2,NULL,NULL);
     gra_x = main_x + GetSystemMetrics(SM_CXFRAME);
     gra_y = main_y + GetSystemMetrics(SM_CYFRAME) +
                      GetSystemMetrics(SM_CYMENU) + 
                      GetSystemMetrics(SM_CYCAPTION);
     valp->lit.vec_va.x_val = (DBfloat)(intx - gra_x - gwinpt->geo.x);
     valp->lit.vec_va.y_val = (DBfloat)(inty - gra_y - gwinpt->geo.y);
     valp->lit.vec_va.z_val = 0.0;
#endif
     return(0);
/*
***Model coordinates ?
*/
     case 3:
     gwinpt = (WPGWIN *)wpwtab[GWIN_MAIN].ptr;
#ifdef V3_X11
     wpgtwp(gwinpt->id.x_id,&gra_x,&gra_y);
     intx = intx - gra_x;
     inty = gwinpt->geo.dy - (inty - gra_y);
#endif
#ifdef WIN32
     msggeo(ms_main,&main_x,&main_y,&dum1,&dum2,NULL,NULL);
     gra_x = main_x + 2*GetSystemMetrics(SM_CXFRAME);
     gra_y = main_y + 2*GetSystemMetrics(SM_CYFRAME) +
                        GetSystemMetrics(SM_CYMENU) +
                      2*GetSystemMetrics(SM_CYCAPTION);
     intx = intx - gra_x - gwinpt->geo.x;
     inty = gwinpt->geo.dy - (inty - gra_y - gwinpt->geo.y);
#endif
     valp->lit.vec_va.x_val = gwinpt->vy.modwin.xmin +
                      (intx - gwinpt->vy.scrwin.xmin)*
                      (gwinpt->vy.modwin.xmax - gwinpt->vy.modwin.xmin)/
                      (gwinpt->vy.scrwin.xmax - gwinpt->vy.scrwin.xmin);
     valp->lit.vec_va.y_val = gwinpt->vy.modwin.ymin +
                      (inty - gwinpt->vy.scrwin.ymin)*
                      (gwinpt->vy.modwin.ymax - gwinpt->vy.modwin.ymin)/
                      (gwinpt->vy.scrwin.ymax - gwinpt->vy.scrwin.ymin);
     valp->lit.vec_va.z_val = 0.0;
     return(0);
/*
***Unknown code.
*/
     default: return(erpush("IN5562",""));
     }

   return(0);
 }

/********************************************************/
/*!******************************************************/

        short evscr(
        PMPARVA  pv[],
        PMLITVA *valp,
        short    np)

/*      Evaluerar funktionen SCREEN.
 *
 *      In: pv   => Pekare till array med parametervrden
 *          valp => Pekare till funktionsvrde.
 *          np   => Antal parametrar.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 18/5/92 J. Kjellander
 *
 ******************************************************!*/

 {
   short   intx; 
   short   inty;
   short   status;
   v2int   win_id;
   PMLITVA litval;

   if ( (status=EXscr(&intx,&inty,&win_id)) != 0 ) return(status);

   valp->lit.vec_va.x_val = (v2float)intx;
   valp->lit.vec_va.y_val = (v2float)inty;
   valp->lit.vec_va.z_val = 0.0;

   if ( np > 0 )
     {
     litval.lit.int_va = win_id;
     inwvar(pv[1].par_ty,pv[1].par_va.lit.adr_va,0,NULL,&litval);
     }

   return(0);
 }

/********************************************************/
/*!******************************************************/

        short evscr2(PMPARVA pv[])

/*      Evaluerar funktionen SCREEN_2.
 *
 *      In: pv => Pekare till array med parametervrden
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 1998-03-23 J. Kjellander
 *
 ******************************************************!*/

 {
   short   status;
   int     ix1,iy1,ix2,iy2,rubmod;
   WPGWIN *gwinpt;
   PMLITVA litval;

/*
***Hmta in 2 koordinater.
*/
   rubmod = pv[4].par_va.lit.int_va;
   if ( rubmod < 0 ) rubmod = 0;
   if ( rubmod > 3 ) rubmod = 3;
#ifdef V3_X11
   status = wpgtsw(&gwinpt,&ix1,&iy1,&ix2,&iy2,rubmod,FALSE);
#endif
#ifdef WIN32
   status = (short)msgtsw(&gwinpt,&ix1,&iy1,&ix2,&iy2,rubmod,FALSE);
#endif
/*
***REJECT eller GOMAIN skall returnera p1 = p2.
*/
   if ( status < 0 )
     {
     ix1 = ix2 = iy1 = iy2 = 0;
     }
/*
***Skriv resultatet till MBS-variabler.
*/
   litval.lit.int_va = gwinpt->id.w_id;
   inwvar(pv[1].par_ty,pv[1].par_va.lit.adr_va,0,NULL,&litval);

   litval.lit.vec_va.x_val = (v2float)ix1;
   litval.lit.vec_va.y_val = (v2float)iy1;
   litval.lit.vec_va.z_val = 0.0;
   inwvar(pv[2].par_ty,pv[2].par_va.lit.adr_va,0,NULL,&litval);

   litval.lit.vec_va.x_val = (v2float)ix2;
   litval.lit.vec_va.y_val = (v2float)iy2;
   litval.lit.vec_va.z_val = 0.0;
   inwvar(pv[3].par_ty,pv[3].par_va.lit.adr_va,0,NULL,&litval);

   return(0);
 }

/********************************************************/
/*!******************************************************/

        short evpos(
        PMPARVA *pv,
        PMLITVA *valp,
        short    np)

/*      Evaluerar funktionen POS.
 *
 *      In: pv   => Pekare till array med parametervrden
 *          valp => Pekare till funktionsvrde.
 *          np   => Antal parametrar.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 18/5/92 J. Kjellander
 *
 ******************************************************!*/

  {
     char   pektkn;
     double fx,fy;
     short   status;
     PMLITVA litva;

/*
***Ls position och pektecken.
*/
    if ( (status=EXpos(&fx,&fy,&pektkn) ) != 0 ) return(status);
/*
***Kopiera pektecknet till PMLITVA.
*/
    litva.lit.str_va[0] = pektkn;
    litva.lit.str_va[1] = '\0';
/*
***Skriv parametervrdet till motsvarande MBS-variabel.
*/
    if ( np == 1 )
      inwvar(pv[1].par_ty,pv[1].par_va.lit.adr_va,0,NULL,&litva);
/*
***Returnera positionen som funktionsvrde.
*/
    valp->lit.vec_va.x_val = (v2float)fx;
    valp->lit.vec_va.y_val = (v2float)fy;
    valp->lit.vec_va.z_val = 0.0;
/*
***Slut.
*/
     return(0);
  }

/********************************************************/
/*!******************************************************/

        short evpos2(PMPARVA pv[])

/*      Evaluerar funktionen POS_2.
 *
 *      In: pv => Pekare till array med parametervrden
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 1998-03-23 J. Kjellander
 *
 *      1998-04-23 REJECT och GOMAIN, J.Kjellander
 *
 ******************************************************!*/

 {
   short   status;
   int     ix1,iy1,ix2,iy2,rubmod;
   v2float mx1,my1,mx2,my2;
   WPGWIN *gwinpt;
   PMLITVA litval;

/*
***Hmta in 2 koordinater.
*/
   rubmod = pv[3].par_va.lit.int_va;
   if ( rubmod < 0 ) rubmod = 0;
   if ( rubmod > 3 ) rubmod = 3;
#ifdef V3_X11
   status = wpgtsw(&gwinpt,&ix1,&iy1,&ix2,&iy2,rubmod,FALSE);
#endif
#ifdef WIN32
   status = (short)msgtsw(&gwinpt,&ix1,&iy1,&ix2,&iy2,rubmod,FALSE);
#endif
/*
***REJECT eller GOMAIN skall returnera p1 = p2.
*/
   if ( status < 0 )
     {
     mx1 = mx2 = my1 = my2 = 0.0;
     }
/*
***Transformera till modellkoordinater.
*/
   else
     {
     mx1 = gwinpt->vy.modwin.xmin + (ix1 - gwinpt->vy.scrwin.xmin)*
                      (gwinpt->vy.modwin.xmax - gwinpt->vy.modwin.xmin)/
                      (gwinpt->vy.scrwin.xmax - gwinpt->vy.scrwin.xmin);
     my1 = gwinpt->vy.modwin.ymin + (iy1 - gwinpt->vy.scrwin.ymin)*
                      (gwinpt->vy.modwin.ymax - gwinpt->vy.modwin.ymin)/
                      (gwinpt->vy.scrwin.ymax - gwinpt->vy.scrwin.ymin);

     mx2 = gwinpt->vy.modwin.xmin + (ix2 - gwinpt->vy.scrwin.xmin)*
                      (gwinpt->vy.modwin.xmax - gwinpt->vy.modwin.xmin)/
                      (gwinpt->vy.scrwin.xmax - gwinpt->vy.scrwin.xmin);
     my2 = gwinpt->vy.modwin.ymin + (iy2 - gwinpt->vy.scrwin.ymin)*
                      (gwinpt->vy.modwin.ymax - gwinpt->vy.modwin.ymin)/
                      (gwinpt->vy.scrwin.ymax - gwinpt->vy.scrwin.ymin);
     }
/*
***Skriv resultatet till MBS-variabler.
*/
   litval.lit.vec_va.x_val = mx1;
   litval.lit.vec_va.y_val = my1;
   litval.lit.vec_va.z_val = 0.0;
   inwvar(pv[1].par_ty,pv[1].par_va.lit.adr_va,0,NULL,&litval);

   litval.lit.vec_va.x_val = mx2;
   litval.lit.vec_va.y_val = my2;
   litval.lit.vec_va.z_val = 0.0;
   inwvar(pv[2].par_ty,pv[2].par_va.lit.adr_va,0,NULL,&litval);

   return(0);
 }

/********************************************************/
/*!******************************************************/

        short evidnt(
        PMPARVA  pv[],
        PMLITVA *valp,
        short    np)

/*      Evaluerar funktionen IDENT.
 *
 *      In: pv   => Pekare till array med parametervrden
 *          valp => Pekare till funktionsvrde.
 *          np   => Antal parametrar.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 18/5/92 J. Kjellander
 *
 ******************************************************!*/

  {
    DBetype typmask;
    bool    end,right;
    short   status;
    PMLITVA val1,val2;

/*
***Typmasken = parameter 1.
*/
    typmask = (DBetype)pv[1].par_va.lit.int_va;
/*
***Anropa exe-rutinen.
*/
    if ( (status=EXidnt(&typmask,valp->lit.ref_va,&end,&right) ) != 0 )
      return( status );
/*
***nde och sida till PMLITVA.
*/
    if ( end == TRUE ) val1.lit.int_va = 1;
    else               val1.lit.int_va = 0;

    if ( right == TRUE ) val2.lit.int_va = 1;
    else                 val2.lit.int_va = 0;
/*
***Ev. skrivning till motsvarande optionella variabler.
*/
    if ( np > 1 ) inwvar(pv[2].par_ty,pv[2].par_va.lit.adr_va,0,NULL,&val1);

    if ( np > 2 ) inwvar(pv[3].par_ty,pv[3].par_va.lit.adr_va,0,NULL,&val2);
/*
***Slut.
*/
    return(0);
  }

/********************************************************/
/*!******************************************************/

        short evidn2(PMPARVA pv[])

/*      Evaluerar funktionen IDENT.
 *
 *      In: pv   => Pekare till array med parametervrden
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      Felkoder: IN5422 = Fr mnga ID:n
 *
 *      (C)microform ab 18/5/92 J. Kjellander
 *
 *      1998-04-03 PRTTYP, J.Kjellander
 *
 ******************************************************!*/

  {
    DBetype typ,typmask,typvek[IGMAXID];
    short   i,nok,status,hitmod,maxref,idant;
    DBptr   la,lavek[IGMAXID];
    PMLITVA litval;
    PMREFVA idvek[MXINIV];

/*
***Typmask, mode och maxref.
*/
    typmask = (DBetype)pv[1].par_va.lit.int_va;
    hitmod  = (short)pv[2].par_va.lit.int_va;
    maxref  = (short)pv[3].par_va.lit.int_va;
/*
***Se till att maxref ligger inom rimliga grnser.
*/
    if      ( maxref < 0 )       maxref = 0;
    else if ( maxref > IGMAXID ) maxref = IGMAXID;
/*
***Hmta ID:n med fnster.
*/
    idant = maxref;
    if ( (typvek[0]=typmask) == PRTTYP ) typvek[0] = ALLTYP;
    status = wpgmlw(lavek,&idant,typvek,hitmod);
    if ( status == REJECT  ||  status == GOMAIN ) idant = -1;
    else if ( status < 0 ) return(status);
/*
***Returnera parameter 4, idmat. Om parter begrts returnerar
***vi bara partid:n.
*/
    nok = 0;
    for ( i=0; i<idant; ++i )
       {
       DBget_id(lavek[i],idvek);
       if ( typmask == PRTTYP )
         {
         idvek[0].p_nextre = NULL;
         DBget_pointer('I',idvek,&la,&typ);
         if ( typ == PRTTYP )
           {
           if ( evwrve(idvek,nok,4,pv) < 0 ) return(erpush("IN5422",""));
         ++nok;
           }
         }
       else
         {
         if ( evwrve(idvek,nok,4,pv) < 0 ) return(erpush("IN5422",""));
       ++nok;
         }
       }
/*
***Returnera parameter 5 antal godknda ID:n.
*/
    litval.lit.int_va = nok;
    inwvar(pv[5].par_ty,pv[5].par_va.lit.adr_va,0,NULL,&litval);
/*
***Slut.
*/
    return(0);
  }

/********************************************************/
/*!******************************************************/

        short evhigh(
        PMPARVA  pv[],
        short    np)

/*      Evaluerar funktionen HIGHLIGHT.
 *
 *      In: pv   => Pekare till array med parametervrden
 *          np   => Antal parametrar.
 *
 *      Ut: Inget.
 *
 *      Felkoder: IN5432 = Storhet finns ej
 *
 *      (C)microform ab 1998-03-23 J. Kjellander
 *
 ******************************************************!*/

 {
   bool    draw;
   DBptr   la;
   DBetype typ;

/*
***Rita eller sudda ?
*/
   if ( pv[1].par_va.lit.int_va == 1 ) draw = TRUE;
   else                                draw = FALSE;

   if ( np == 1 )
     {
     gphgal(draw);
     }
   else
     {
     if ( DBget_pointer('I',pv[2].par_va.lit.ref_va,&la,&typ) < 0 )
       return(erpush("IN5432",""));
     wphgen(GWIN_ALL,la,draw);
     }

   return(0);
 }

/********************************************************/
