/**********************************************************************
*
*    eval6.c
*    =======
*
*    This file is part of the VARKON Program Module Library.
*    URL: http://www.varkon.com
*
*    evpofr();     Evaluerar POI_FREE/PROJ
*
*    evlifr();     Evaluerar LIN_FREE/PROJ
*    evliof();     Evaluerar LIN_OFFS
*    evlian();     Evaluerar LIN_ANG
*    evlit1();     Evaluerar LIN_TAN1
*    evlit2();     Evaluerar LIN_TAN2
*    evlipe();     Evaluerar LIN_PERP
*
*    evarfr();     Evaluerar ARC_1POS
*    evar2p();     Evaluerar ARC_2POS
*    evar3p();     Evaluerar ARC_3POS
*    evarof();     Evaluerar ARC_OFFS
*    evarfl();     Evaluerar ARC_FIL
*
*    vcufr();      Evaluerar CUR_FREE/PROJ
*    evcurs();     Evaluerar CUR_SPLINE
*    evcusa();     Evaluerar CUR_SPLARR
*    evcurc();     Evaluerar CUR_CONIC
*    evcuca();     Evaluerar CUR_CONARR
*    evcomp();     Evaluerar CUR_COMP
*    evcuro();     Evaluerar CUR_OFFS
*    evcurt();     Evaluerar CUR_TRIM
*    evcuru();     Evaluerar CUR_USRDEF
*
*    evcs3p();     Evaluerar CSYS_3P
*    evcs1p();     Evaluerar CSYS_1P
*    
*    evgrp();      Evaluerar GROUP
*
*    This library is free software; you can redistribute it and/or
*    modify it under the terms of the GNU Library General Public
*    License as published by the Free Software Foundation; either
*    version 2 of the License, or (at your option) any later version.
*
*    This library is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*    Library General Public License for more details.
*
*    You should have received a copy of the GNU Library General Public
*    License along with this library; if not, write to the Free
*    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
*    (C)Microform AB 1984-1999, Johan Kjellander, johan@microform.se
*
***********************************************************************/

#include "../../DB/include/DB.h"
#include "../../IG/include/IG.h"
#include "../include/mbsrout.h"
#include "../../EX/include/EX.h"

/*!*******************************************************
*
*      eval6.c
*      =======
*
*      Denna fil innehller:
*
*
*******************************************************!*/

extern short  modtyp;

/*!******************************************************/

        short evpofr(
        V2REFVA *ident,
        PMPARVA  pv[],
        pmroutn  routine,
        V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren POJ_(FREE)(PROJ).
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          routine  => FREE/PROJ.
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 26/12/86 J. Kjellander
 *
 ******************************************************!*/

{

/*
***Exekvera POI_FREE alt. POI_PROJ.
*/
   if ( routine == VPOI )
     {
     return(EXpofr( ident, (DBVector *)&pv[1].par_va.lit.vec_va, npblockp));
     }
   else
     {
     return(EXpopr( ident, (DBVector *)&pv[1].par_va.lit.vec_va, npblockp));
     }
}

/********************************************************/
/*!******************************************************/

        short evlifr(
        V2REFVA *ident,
        PMPARVA  pv[],
        pmroutn  routine,
        V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren LIN_(FREE)(PROJ).
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          routine  => FREE/PROJ.
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 26/12/86 J. Kjellander
 *
 ******************************************************!*/

{
/*
***Exekvera LIN_FREE alt. LIN_PROJ.
*/
   if ( routine == VLINF )
     {
     return(EXlifr( ident, (DBVector *)&pv[1].par_va.lit.vec_va,
                           (DBVector *)&pv[2].par_va.lit.vec_va, npblockp));
     }
   else
     {
     return(EXlipr( ident, (DBVector *)&pv[1].par_va.lit.vec_va,
                           (DBVector *)&pv[2].par_va.lit.vec_va, npblockp));
     }
}

/********************************************************/
/*!******************************************************/

        short evliof(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren LIN_OFFS.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 26/12/86 J. Kjellander
 *
 ******************************************************!*/

{
/*
***Exekvera LIN_OFFS.
*/
   return(EXliof( ident, pv[1].par_va.lit.ref_va,
                         pv[2].par_va.lit.float_va, npblockp));

}

/********************************************************/
/*!******************************************************/

        short evlian(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren LIN_ANG.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 26/12/86 J. Kjellander
 *
 ******************************************************!*/

{
/*
***Exekvera LIN_ANG.
*/
   return(EXlipv( ident, (DBVector *)&pv[1].par_va.lit.vec_va,
                          pv[2].par_va.lit.float_va,
                          pv[3].par_va.lit.float_va, npblockp));
}

/********************************************************/
/*!******************************************************/

        short evlit1(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren LIN_TAN1.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 26/12/86 J. Kjellander
 *
 ******************************************************!*/

{
/*
***Exekvera LIN_TAN1.
*/
   return(EXlipt( ident, (DBVector *)&pv[1].par_va.lit.vec_va,
                          pv[2].par_va.lit.ref_va,
                   (short)pv[3].par_va.lit.int_va, npblockp));
}

/********************************************************/
/*!******************************************************/

       short evlit2(
       V2REFVA *ident,
       PMPARVA  pv[],
       V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren LIN_TAN2.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 26/12/86 J. Kjellander
 *
 ******************************************************!*/

{
/*
***Exekvera LIN_TAN2.
*/
   return(EXli2t( ident,  pv[1].par_va.lit.ref_va,
                          pv[2].par_va.lit.ref_va,
                   (short)pv[3].par_va.lit.int_va, npblockp));
}

/********************************************************/
/*!******************************************************/

        short evlipe(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren LIN_PERP.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 24/4/87  R. Svedin
 *
 ******************************************************!*/

{
/*
***Exekvera LIN_PERP.
*/
   return(EXlipe( ident, (DBVector *)&pv[1].par_va.lit.vec_va,
                          pv[2].par_va.lit.ref_va,
                          pv[3].par_va.lit.float_va, npblockp));
}

/********************************************************/
/*!******************************************************/

       short evarfr(
       V2REFVA *ident,
       PMPARVA  pv[],
       V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren ARC_1POS.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 26/12/86 J. Kjellander
 *
 ******************************************************!*/

{
/*
***Exekvera ARC_1POS.
*/
   return(EXarfr( ident, (DBVector *)&pv[1].par_va.lit.vec_va,
                          pv[2].par_va.lit.float_va,
                          pv[3].par_va.lit.float_va,
                          pv[4].par_va.lit.float_va,
                          npblockp));
}
/********************************************************/
/*!******************************************************/

       short evar2p(
       V2REFVA *ident,
       PMPARVA  pv[],
       V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren ARC_2POS.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 26/12/86 J. Kjellander
 *
 ******************************************************!*/

{
/*
***Exekvera ARC_2POS.
*/
   return(EXar2p( ident, (DBVector *)&pv[1].par_va.lit.vec_va,
                         (DBVector *)&pv[2].par_va.lit.vec_va,
                          pv[3].par_va.lit.float_va,
                          npblockp));
}

/********************************************************/
/*!******************************************************/

       short evar3p(
       V2REFVA *ident,
       PMPARVA  pv[],
       V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren ARC_3POS.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 26/12/86 J. Kjellander
 *
 ******************************************************!*/

{
/*
***Exekvera ARC_3POS.
*/
   return(EXar3p( ident, (DBVector *)&pv[1].par_va.lit.vec_va,
                         (DBVector *)&pv[2].par_va.lit.vec_va,
                         (DBVector *)&pv[3].par_va.lit.vec_va,
                          npblockp));
}

/********************************************************/
/*!******************************************************/

        short evarof(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren ARC_OFFS.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 26/12/86 J. Kjellander
 *
 ******************************************************!*/

{
/*
***Exekvera ARC_OFS.
*/
   return(EXarof( ident,  pv[1].par_va.lit.ref_va,
                          pv[2].par_va.lit.float_va,
                          npblockp));
}

/********************************************************/
/*!******************************************************/

        short evarfl(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren ARC_FIL.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 26/12/86 J. Kjellander
 *
 ******************************************************!*/

{
/*
***Exekvera ARC_FIL.
*/
   return(EXarfl( ident,  pv[1].par_va.lit.ref_va,
                          pv[2].par_va.lit.ref_va,
                          pv[3].par_va.lit.float_va,
                   (short)pv[4].par_va.lit.int_va,
                          npblockp));
}

/********************************************************/
/*!******************************************************/

        short evcufr(
        V2REFVA *ident,
        PMPARVA  pv[],
        pmroutn  routine,
        V2NAPA  *npblockp,
        short    npar)

/*      Evaluerar geometri-proceduren CUR_FREE/PROJ.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          routine  => FREE/PROJ.
 *          npblockp => Pekare till namnparameterblock.
 *          npar     => Antal parametrar.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 26/12/86 J. Kjellander
 *
 ******************************************************!*/

{
   short   npoi = 0;  /* Number of points */ 
   short   i;         /* loop variable */
   V2VECVA poiarr[V2PARMAX];  /* point array */
   V2VECVA tanarr[V2PARMAX];  /* tangent array */

/*
***Skapa poiarr och tanarr.
*/
   for( i = 1; i <= npar; i+=2 )
     { 
     poiarr[ npoi ].x_val = pv[ i ].par_va.lit.vec_va.x_val;
     poiarr[ npoi ].y_val = pv[ i ].par_va.lit.vec_va.y_val;
     poiarr[ npoi ].z_val = pv[ i ].par_va.lit.vec_va.z_val;
     tanarr[ npoi ].x_val = pv[ i+1 ].par_va.lit.vec_va.x_val;
     tanarr[ npoi ].y_val = pv[ i+1 ].par_va.lit.vec_va.y_val;
     tanarr[ npoi ].z_val = pv[ i+1 ].par_va.lit.vec_va.z_val;
     npoi++;
     }
/*
***Exekvera CUR_FREE alt. CUR_PROJ.
*/
   if ( routine == VCURF )
     {
     return(EXcufr( ident, (DBVector *)poiarr, (DBVector *)tanarr, npoi, npblockp));
     }
   else
     {
     return(EXcupr( ident, (DBVector *)poiarr, (DBVector *)tanarr, npoi, npblockp));
     }
}

/********************************************************/
/*!******************************************************/

        short evcurs(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp,
        short    npar)

/*      Evaluerar geometri-proceduren CUR_SPLINE.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *          npar     => Antal parametrar.
 *
 *      Ut: Inget.
 *
 *      Felkoder: IN5002 = Felaktig typ av kurva
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 15/11/91 J. Kjellander
 *
 ******************************************************!*/

{
   DBint   npoi;                  /* Number of points */ 
   int     i;                     /* Loop variable */
   char   *ptyp;                  /* Ferguson/Chord.... */
   DBint   type;                  /* Type code */
   V2VECVA poiarr[V2PARMAX];      /* Point array */
   V2VECVA tanarr[V2PARMAX];      /* Tangent array */

/*
***What type ?
*/
   ptyp = pv[1].par_va.lit.str_va;

   if      ( sticmp(ptyp,"ferguson")  == 0 ) type = 1;
   else if ( sticmp(ptyp,"chord")     == 0 ) type = 2;
   else if ( sticmp(ptyp,"stiffness") == 0 ) type = 3;
   else if ( sticmp(ptyp,"free")      == 0 ) type = 4;
   else if ( sticmp(ptyp,"proj")      == 0 ) type = 5;
   else return(erpush("IN5002",ptyp));
/*
***Skapa poiarr och tanarr.
*/
   npoi = 0;

   for( i = 2; i <= npar; i+=2 )
     { 
     poiarr[ npoi ].x_val = pv[ i ].par_va.lit.vec_va.x_val;
     poiarr[ npoi ].y_val = pv[ i ].par_va.lit.vec_va.y_val;
     poiarr[ npoi ].z_val = pv[ i ].par_va.lit.vec_va.z_val;
     tanarr[ npoi ].x_val = pv[ i+1 ].par_va.lit.vec_va.x_val;
     tanarr[ npoi ].y_val = pv[ i+1 ].par_va.lit.vec_va.y_val;
     tanarr[ npoi ].z_val = pv[ i+1 ].par_va.lit.vec_va.z_val;
     npoi++;
     }
/*
***Exekvera.
*/
   return(EXcurs(ident,type,(DBVector *)poiarr,
                            (DBVector *)tanarr,npoi,npblockp));
}

/********************************************************/
/*!******************************************************/

        short evcusa(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp,
        short    np)

/*      Evaluerar geometri-proceduren CUR_SPLARR.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *          np       => Antal parametrar.
 *
 *      Ut: Inget.
 *
 *      Felkoder: IN5002 = Felaktig typ av kurva
 *                IN5132 = Antal positioner < 2
 *                IN5142 = Deklarerad storlek < npos
 *                IN5152 = Kan ej allokera minne
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 28/2/95 J. Kjellander
 *
 ******************************************************!*/

{
   char     *ptyp;
   char      errbuf[81];
   short     status;
   int       dekl_dim,vecsiz;
   DBint     type,npos,i;
   DBint     posadr,tanadr = 0;
   STTYTBL   typtbl;
   STARRTY   arrtyp;
   DBVector *ppek,*tpek;
   PMLITVA   val;

/*
***What type ?
*/
   ptyp = pv[1].par_va.lit.str_va;

   if      ( sticmp(ptyp,"ferguson")  == 0 ) type = 1;
   else if ( sticmp(ptyp,"chord")     == 0 ) type = 2;
   else if ( sticmp(ptyp,"stiffness") == 0 ) type = 3;
   else if ( sticmp(ptyp,"free")      == 0 ) type = 4;
   else if ( sticmp(ptyp,"proj")      == 0 ) type = 5;
   else return(erpush("IN5002",ptyp));
/*
***Hur mnga positioner.
*/
   npos = pv[2].par_va.lit.int_va;
/*
***Kolla att antalet positioner r rimligt.
*/
   if ( npos < 2 ) return(erpush("IN5132",""));
/*
***Kolla att deklarerad dimension p MBS-arrayer r tillrcklig fr
***npos positioner och tangenter.
*/
   posadr = pv[3].par_va.lit.adr_va;
   strtyp(pv[3].par_ty,&typtbl);
   strarr(typtbl.arr_ty,&arrtyp);
   dekl_dim = arrtyp.up_arr - arrtyp.low_arr + 1;
   if ( dekl_dim < npos ) return(erpush("IN5142",""));

   if ( np == 4 )
     {
     tanadr = pv[4].par_va.lit.adr_va;
     strtyp(pv[4].par_ty,&typtbl);
     strarr(typtbl.arr_ty,&arrtyp);
     dekl_dim = arrtyp.up_arr - arrtyp.low_arr + 1;
     if ( dekl_dim < npos ) return(erpush("IN5142",""));
     }
/*
***Allokera minne fr positioner och tangenter.
*/
   if ( (ppek=(DBVector *)v3mall(npos*sizeof(DBVector),"evcusa")) == NULL )
     {
     sprintf(errbuf,"%d",npos);
     return(erpush("IN5152",errbuf));
     }

   if ( (tpek=(DBVector *)v3mall(npos*sizeof(DBVector),"evcusa")) == NULL )
     {
     sprintf(errbuf,"%d",npos);
     return(erpush("IN5152",errbuf));
     }
/*
***vecsiz = storleken p en VECTOR.
*/
   strtyp(arrtyp.base_arr,&typtbl);
   vecsiz = typtbl.size_ty;
/*
***Kopiera positioner frn RTS till allokerad area.
*/
   for ( i=0; i<npos; ++i )
     {
     ingval(posadr+i*vecsiz,arrtyp.base_arr,FALSE,&val);
     (ppek+i)->x_gm = val.lit.vec_va.x_val;
     (ppek+i)->y_gm = val.lit.vec_va.y_val;
     (ppek+i)->z_gm = val.lit.vec_va.z_val;
     if ( np == 4 )
       {
       ingval(tanadr+i*vecsiz,arrtyp.base_arr,FALSE,&val);
       (tpek+i)->x_gm = val.lit.vec_va.x_val;
       (tpek+i)->y_gm = val.lit.vec_va.y_val;
       (tpek+i)->z_gm = val.lit.vec_va.z_val;
       }
     else
       {
       (tpek+i)->x_gm = (tpek+i)->y_gm = (tpek+i)->z_gm = 0.0;
       }
     }
/*
***Anropa exe-rutinen.
*/
   status = EXcurs(ident,type,ppek,tpek,npos,npblockp);
/*
***Deallokera minne.
*/
   v3free(ppek,"evcusa");
   v3free(tpek,"evcusa");

   return(status);
}

/********************************************************/
/*!******************************************************/

        short evcurc(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp,
        short    npar)

/*      Evaluerar geometri-proceduren CUR_CONIC.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *          npar     => Antal parametrar.
 *
 *      Ut: Inget.
 *
 *      Felkoder: IN5002 = Felaktig typ av kurva
 *                IN5012 = P-vrde < 0
 *                IN5022 = P-vrde felaktig typ
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 12/11/91 J. Kjellander
 *
 ******************************************************!*/

{
   short   npoi = 0;              /* Number of points */ 
   short   i,j;                   /* loop variable */
   char   *typ;                   /* FREE/PROJ */
   bool    proj;                  /* Projicera ja/nej */
   V2VECVA poiarr[V2PARMAX];      /* point array */
   V2VECVA tanarr[V2PARMAX];      /* tangent array */
   V2VECVA iparr [V2PARMAX];      /* intermediate point array */
   gmflt   pvarr [V2PARMAX];      /* P-value array */
   char    errbuf[20];            /* Felbuffert */

/*
***FREE eller PROJ.
*/
   typ = pv[1].par_va.lit.str_va;
   if      ( sticmp(typ,"free") == 0 ) proj = FALSE;
   else if ( sticmp(typ,"proj") == 0 ) proj = TRUE;
   else return(erpush("IN5002",typ));
/*
***Skapa poiarr och tanarr.
*/
   for( j=0,i=2; i<npar; j++,i+=3 )
     { 
     poiarr[j].x_val = pv[i  ].par_va.lit.vec_va.x_val;
     poiarr[j].y_val = pv[i  ].par_va.lit.vec_va.y_val;
     poiarr[j].z_val = pv[i  ].par_va.lit.vec_va.z_val;
     tanarr[j].x_val = pv[i+1].par_va.lit.vec_va.x_val;
     tanarr[j].y_val = pv[i+1].par_va.lit.vec_va.y_val;
     tanarr[j].z_val = pv[i+1].par_va.lit.vec_va.z_val;
     npoi++;
     }
/*
***Skapa pvarr och iparr.
*/
   for( j=0,i=4; i<npar; j++,i+=3 )
     { 
     switch ( pv[i].par_va.lit_type )
       {
       case C_INT_VA:
       iparr[j].x_val = 0.0;
       iparr[j].y_val = 0.0;
       iparr[j].z_val = 0.0;
       pvarr[j] = (gmflt)pv[i].par_va.lit.int_va;
       if ( pvarr[j] < 0.0 )
         {
         sprintf(errbuf,"%d",(int)(j+1));
         return(erpush("IN5012",errbuf));
         }
       break;

       case C_FLO_VA:
       iparr[j].x_val = 0.0;
       iparr[j].y_val = 0.0;
       iparr[j].z_val = 0.0;
       pvarr[j] = (gmflt)pv[i].par_va.lit.float_va;
       if ( pvarr[j] < 0.0 )
         {
         sprintf(errbuf,"%d",(int)(j+1));
         return(erpush("IN5012",errbuf));
         }
       break;

       case C_VEC_VA:
       iparr[j].x_val = pv[i].par_va.lit.vec_va.x_val;
       iparr[j].y_val = pv[i].par_va.lit.vec_va.y_val;
       iparr[j].z_val = pv[i].par_va.lit.vec_va.z_val;
       pvarr[j] = -1.0;
       break;

       default:
       return(erpush("IN5022",""));
       }
     }
/*
***Exekvera CUR_CONIC.
*/
   return(EXcurc(ident,proj,(DBVector *)poiarr,(DBVector *)tanarr,
                            (DBVector *)iparr,pvarr,npoi,npblockp));
}

/********************************************************/
/*!******************************************************/

        short evcuca(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren CUR_CONARR.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *          npar     => Antal parametrar.
 *
 *      Ut: Inget.
 *
 *      Felkoder: IN5002 = Felaktig typ av kurva
 *                IN5162 = Antal positioner < 2
 *                IN5172 = Deklarerad storlek < npos
 *                IN5182 = Kan ej allokera minne
 *                IN5192 = Typ skall vara m/M/p/P
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 2/3/95 J. Kjellander
 *
 ******************************************************!*/

{
   bool    proj;
   char   *typ;
   char    errbuf[81];
   short   status;
   int     dekl_dim,vecsiz,strsiz,fltsiz;
   gmint   npos,i;
   DBint   posadr,tanadr,metadr,pvadr,mpadr;
   gmflt  *pvpek;
   STTYTBL typtbl;
   STARRTY postyp,tantyp,mettyp,pvtyp,mptyp;
   DBVector  *ppek,*tpek,*mppek;
   PMLITVA val;

/*
***FREE eller PROJ.
*/
   typ = pv[1].par_va.lit.str_va;
   if      ( sticmp(typ,"free") == 0 ) proj = FALSE;
   else if ( sticmp(typ,"proj") == 0 ) proj = TRUE;
   else return(erpush("IN5002",typ));
/*
***Hur mnga positioner.
*/
   npos = pv[2].par_va.lit.int_va;
/*
***Kolla att antalet positioner r rimligt.
*/
   if ( npos < 2 ) return(erpush("IN5162",""));
/*
***Kolla att deklarerad dimension p MBS-arrayer r tillrcklig fr
***npos positioner, tangenter osv.
*/
   posadr = pv[3].par_va.lit.adr_va;
   strtyp(pv[3].par_ty,&typtbl);
   strarr(typtbl.arr_ty,&postyp);
   dekl_dim = postyp.up_arr - postyp.low_arr + 1;
   if ( dekl_dim < npos ) return(erpush("IN5172",""));

   strtyp(postyp.base_arr,&typtbl);
   vecsiz = typtbl.size_ty;

   tanadr = pv[4].par_va.lit.adr_va;
   strtyp(pv[4].par_ty,&typtbl);
   strarr(typtbl.arr_ty,&tantyp);
   dekl_dim = tantyp.up_arr - tantyp.low_arr + 1;
   if ( dekl_dim < npos ) return(erpush("IN5172",""));

   metadr = pv[5].par_va.lit.adr_va;
   strtyp(pv[5].par_ty,&typtbl);
   strarr(typtbl.arr_ty,&mettyp);
   dekl_dim = mettyp.up_arr - mettyp.low_arr + 1;
   if ( dekl_dim < npos-1 ) return(erpush("IN5172",""));

   strtyp(mettyp.base_arr,&typtbl);
   strsiz = typtbl.size_ty;

   pvadr = pv[6].par_va.lit.adr_va;
   strtyp(pv[6].par_ty,&typtbl);
   strarr(typtbl.arr_ty,&pvtyp);
   dekl_dim = pvtyp.up_arr - pvtyp.low_arr + 1;
   if ( dekl_dim < npos-1 ) return(erpush("IN5172",""));

   strtyp(pvtyp.base_arr,&typtbl);
   fltsiz = typtbl.size_ty;

   mpadr = pv[7].par_va.lit.adr_va;
   strtyp(pv[7].par_ty,&typtbl);
   strarr(typtbl.arr_ty,&mptyp);
   dekl_dim = mptyp.up_arr - mptyp.low_arr + 1;
   if ( dekl_dim < npos-1 ) return(erpush("IN5172",""));
/*
***Allokera minne.
*/
   if ( (ppek=(DBVector *)v3mall(npos*sizeof(DBVector),"evcuca")) == NULL )
     {
     sprintf(errbuf,"%d",npos);
     return(erpush("IN5182",errbuf));
     }

   if ( (tpek=(DBVector *)v3mall(npos*sizeof(DBVector),"evcuca")) == NULL )
     {
     sprintf(errbuf,"%d",npos);
     return(erpush("IN5182",errbuf));
     }

   if ( (pvpek=(gmflt *)v3mall(npos*sizeof(gmflt),"evcuca")) == NULL )
     {
     sprintf(errbuf,"%d",npos);
     return(erpush("IN5182",errbuf));
     }

   if ( (mppek=(DBVector *)v3mall(npos*sizeof(DBVector),"evcuca")) == NULL )
     {
     sprintf(errbuf,"%d",npos);
     return(erpush("IN5182",errbuf));
     }
/*
***Kopiera frn RTS till allokerade areor.
*/
   for ( i=0; i<npos; ++i )
     {
     ingval(posadr+i*vecsiz,postyp.base_arr,FALSE,&val);
     (ppek+i)->x_gm = val.lit.vec_va.x_val;
     (ppek+i)->y_gm = val.lit.vec_va.y_val;
     (ppek+i)->z_gm = val.lit.vec_va.z_val;

     ingval(tanadr+i*vecsiz,tantyp.base_arr,FALSE,&val);
     (tpek+i)->x_gm = val.lit.vec_va.x_val;
     (tpek+i)->y_gm = val.lit.vec_va.y_val;
     (tpek+i)->z_gm = val.lit.vec_va.z_val;
     }

   for ( i=0; i<npos-1; ++i )
     {
     ingval(metadr+i*strsiz,mettyp.base_arr,FALSE,&val);
     if ( sticmp(val.lit.str_va,"p") == 0 )
       {
       ingval(pvadr+i*fltsiz,pvtyp.base_arr,FALSE,&val);
      *(pvpek+i) = val.lit.float_va;
       (mppek+i)->x_gm =
       (mppek+i)->y_gm =
       (mppek+i)->z_gm = 0.0;
       }
     else if ( sticmp(val.lit.str_va,"m") == 0 )
       {
       ingval(mpadr+i*vecsiz,mptyp.base_arr,FALSE,&val);
      *(pvpek+i) = -1.0;
       (mppek+i)->x_gm = val.lit.vec_va.x_val;
       (mppek+i)->y_gm = val.lit.vec_va.y_val;
       (mppek+i)->z_gm = val.lit.vec_va.z_val;
       }
     else
       {
       status = erpush("IN5192","");
       goto exit;
       }
     }
/*
***Anropa exe-rutinen.
*/
   status = EXcurc(ident,proj,ppek,tpek,mppek,pvpek,(short)npos,npblockp);
/*
***Deallokera minne.
*/
exit:
   v3free(ppek,"evcuca");
   v3free(tpek,"evcuca");
   v3free(pvpek,"evcuca");
   v3free(mppek,"evcuca");

   return(status);
}

/********************************************************/
/*!******************************************************/

        short evcuro(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren CUR_OFFS.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 15/11/91 J. Kjellander
 *
 ******************************************************!*/

{
/*
***Exekvera CUR_OFFS.
*/
   return(EXcuro(ident,pv[1].par_va.lit.ref_va,
                        pv[2].par_va.lit.float_va,
                        npblockp));
}

/********************************************************/
/*!******************************************************/

        short evcomp(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp,
        short    parcount)

/*      Evaluerar geometri-proceduren CUR_COMP.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *          parcount => Antal parametrar.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 26/12/86 J. Kjellander
 *
 ******************************************************!*/

{
   short i;                        /* loop variable */
   short  nref = 0;                /* Number of referenses */ 
   V2REFVA  refarr[ GMMXCO + 1 ];  /* referens array */

/*
***Skapa refarr.
*/
   for ( i = 1 ; i <= parcount ; i++ )
     {
     refarr[ nref ].seq_val = pv[i].par_va.lit.ref_va[ 0 ].seq_val;
     refarr[ nref ].ord_val = pv[i].par_va.lit.ref_va[ 0 ].ord_val;
     refarr[ nref ].p_nextre = pv[i].par_va.lit.ref_va[ 0 ].p_nextre;
     nref++;
     }
/*
***Exekvera COMP.
*/
   return(EXcomp( ident, refarr, nref, npblockp));

}

/********************************************************/
/*!******************************************************/

        short evcurt(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp)

/*      Evaluerar geometri-proceduren CUR_TRIM.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 15/11/91 J. Kjellander
 *
 ******************************************************!*/

{
/*
***Exekvera CUR_TRIM.
*/
    return(EXcurt(ident,pv[1].par_va.lit.ref_va,
                        pv[2].par_va.lit.float_va,
                        pv[3].par_va.lit.float_va,
                        npblockp));
}

/********************************************************/
/*!******************************************************/

        short evcuru(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp,
        short    np)

/*      Evaluerar geometri-proceduren CUR_USDEF.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *          np       => Antal parametrar.
 *
 *      Ut: Inget.
 *
 *      Felkoder: IN5032 = nseg < 0
 *                IN5042 = Kan ej mallokera minne fr nseg GMSEG
 *                IN5052 = nseg > Dimensionen p 4X4-variabeln.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 15/11/91 J. Kjellander
 *
 *      2/10/92    Slutputs, J. Kjellander
 *      5/3/93     CUB_SEG, J. Kjellander
 *      14/6/93    Dynamiska segment, J. Kjellander
 *      1/6/94     VAX-bug sl=0, J. Kjellander
 *      1997-12-18 Yt-ID, J.Kjellander
 *
 ******************************************************!*/

{
   short    nseg,status;
   int      up_bound,i;
   DBint    valadr;
   int      segsiz,radsiz,fltsiz;
   PMREFVA *surid;
   PMLITVA  fval;
   STTYTBL  typtbl;
   STARRTY  arrtyp;
   GMSEG   *segptr,*segdat;


/*
***v3c kollar att indata r:
***  Par 1: INT          Antal segment, ett vrde.
***  Par 2: FLOAT(?,4,4) Segmentdata, en adress.
***Antal segment ?.
*/
   nseg = (short)pv[1].par_va.lit.int_va;
   if ( nseg < 1 ) return(erpush("IN5032",""));
/*
***Allokera minne fr nseg segment.
*/
   segptr = segdat = DBcreate_segments((int)nseg);
   if ( segptr == NULL ) return(erpush("IN5042",""));
/*
***Kolla att dimensionen p MBS-segdata-arrayen r tillrcklig fr
***nseg segment, dvs att den r deklarerad minst (nseg,4,4), detta
***kan ju inte kollas av v3c.
*/
   strtyp(pv[2].par_ty,&typtbl);
   strarr(typtbl.arr_ty,&arrtyp);
   up_bound = arrtyp.up_arr - arrtyp.low_arr + 1;
   if ( up_bound < nseg )
     {
     status = erpush("IN5052","");
     goto end;
     }
/*
***Berkna div. RTS-offset.
***segsiz = storleken p en FLOAT (4,4)     Normalt 128 bytes.
***radsiz = storleken p en FLOAT (4)       Normalt 32  bytes.
***fltsiz = storleken p en FLOAT           Normalt 8   bytes.
*/
   strtyp(arrtyp.base_arr,&typtbl);
   segsiz = typtbl.size_ty;

   strarr(typtbl.arr_ty,&arrtyp);
   strtyp(arrtyp.base_arr,&typtbl);
   radsiz = typtbl.size_ty;

   strarr(typtbl.arr_ty,&arrtyp);
   strtyp(arrtyp.base_arr,&typtbl);
   fltsiz = typtbl.size_ty;
/*
***Kopiera 4X4-matriserna till GMSEG, och stt offset = 0.0.
*/
   valadr = pv[2].par_va.lit.adr_va;

   for ( i=0; i<nseg; ++i )
     {
     ingval(valadr,arrtyp.base_arr,FALSE,&fval);
     segptr->c0x = fval.lit.float_va;
     ingval(valadr+fltsiz,arrtyp.base_arr,FALSE,&fval);
     segptr->c0y = fval.lit.float_va;
     ingval(valadr+2*fltsiz,arrtyp.base_arr,FALSE,&fval);
     segptr->c0z = fval.lit.float_va;
     ingval(valadr+3*fltsiz,arrtyp.base_arr,FALSE,&fval);
     segptr->c0  = fval.lit.float_va;

     ingval(valadr+radsiz,arrtyp.base_arr,FALSE,&fval);
     segptr->c1x = fval.lit.float_va;
     ingval(valadr+radsiz+fltsiz,arrtyp.base_arr,FALSE,&fval);
     segptr->c1y = fval.lit.float_va;
     ingval(valadr+radsiz+2*fltsiz,arrtyp.base_arr,FALSE,&fval);
     segptr->c1z = fval.lit.float_va;
     ingval(valadr+radsiz+3*fltsiz,arrtyp.base_arr,FALSE,&fval);
     segptr->c1  = fval.lit.float_va;

     ingval(valadr+2*radsiz,arrtyp.base_arr,FALSE,&fval);
     segptr->c2x = fval.lit.float_va;
     ingval(valadr+2*radsiz+fltsiz,arrtyp.base_arr,FALSE,&fval);
     segptr->c2y = fval.lit.float_va;
     ingval(valadr+2*radsiz+2*fltsiz,arrtyp.base_arr,FALSE,&fval);
     segptr->c2z = fval.lit.float_va;
     ingval(valadr+2*radsiz+3*fltsiz,arrtyp.base_arr,FALSE,&fval);
     segptr->c2  = fval.lit.float_va;

     ingval(valadr+3*radsiz,arrtyp.base_arr,FALSE,&fval);
     segptr->c3x = fval.lit.float_va;
     ingval(valadr+3*radsiz+fltsiz,arrtyp.base_arr,FALSE,&fval);
     segptr->c3y = fval.lit.float_va;
     ingval(valadr+3*radsiz+2*fltsiz,arrtyp.base_arr,FALSE,&fval);
     segptr->c3z = fval.lit.float_va;
     ingval(valadr+3*radsiz+3*fltsiz,arrtyp.base_arr,FALSE,&fval);
     segptr->c3  = fval.lit.float_va;

     segptr->typ = CUB_SEG;
     segptr->ofs = 0.0;
     segptr->sl  = 0.0;

     valadr += segsiz;
     segptr++;
     }
/*
***Har yt-id skickats med ?
*/
   if ( np == 3 ) surid = pv[3].par_va.lit.ref_va;
   else           surid = NULL;
/*
***Exekvera CUR_USDEF.
*/
   status = EXcuru(ident,nseg,segdat,surid,npblockp);
/*
***Slut.
*/
end:
   DBfree_segments(segdat);
   return(status);
}

/********************************************************/
/*!******************************************************/

        short    evcs3p(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp,
        short    np)

/*      Evaluerar geometri-proceduren CSYS_3P.
 *
 *      In: ident    = Storhetens ID.
 *          pv       = Pekare till array med parametervrden
 *          npblockp = Pekare till namnparameterblock.
 *          np       = Antal parametrar.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 26/12/86 J. Kjellander
 *
 *       1996-05-28 Optionell Y-axel, J.Kjellander
 *
 ******************************************************!*/

{
   DBVector *pyaxel;

/*
***Har vi ftt en Y-axel eller har den utelmnats.
*/
   if ( np ==  4 ) pyaxel = (DBVector *)&pv[4].par_va.lit.vec_va;
   else            pyaxel =  NULL;
/*
***Exekvera CSYS_3P.
*/
   return(EXcs3p( ident, pv[1].par_va.lit.str_va,
                        (DBVector *)&pv[2].par_va.lit.vec_va,
                        (DBVector *)&pv[3].par_va.lit.vec_va,
                         pyaxel,
                         npblockp));
}

/********************************************************/
/*!******************************************************/

        short evcs1p(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp,
        short    np)

/*      Evaluerar geometri-proceduren CSYS_1P.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          np       => Antal parametrar.
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 30/9/87 J. Kjellander
 *
 ******************************************************!*/

{
   gmflt v1,v2,v3;

/*
***Exekvera CSYS_1P.
*/
   if ( modtyp == 3 )
     {
     if ( np == 2 )
       {
       v1 = v2 = v3 = 0.0;
       }
     else if ( np == 3 )
       {
       v1 = pv[3].par_va.lit.float_va;
       v2 = v3 = 0.0;
       }
     else if ( np == 4 )
       {
       v1 = pv[3].par_va.lit.float_va;
       v2 = pv[4].par_va.lit.float_va;
       v3 = 0.0;
       }
     else
       {
       v1 = pv[3].par_va.lit.float_va;
       v2 = pv[4].par_va.lit.float_va;
       v3 = pv[5].par_va.lit.float_va;
       }
     }
/*
***2D-fallet.
*/
   else
     {
     if ( np < 5 )
       {
       v1 = v2 = v3 = 0.0;
       }
     else
       {
       v3 = pv[5].par_va.lit.float_va;
       v1 = v2 = 0.0;
       }
     }
/*
***Exekvera.
*/
   return(EXcs1p( ident, pv[1].par_va.lit.str_va,
                        (DBVector *)&pv[2].par_va.lit.vec_va,
                         v1,v2,v3, npblockp));
}

/********************************************************/
/*!******************************************************/

        short evgrp(
        V2REFVA *ident,
        PMPARVA  pv[],
        V2NAPA  *npblockp,
        short    parcount)

/*      Evaluerar geometri-proceduren GROUP.
 *
 *      In: ident    => Storhetens ID.
 *          pv       => Pekare till array med parametervrden
 *          npblockp => Pekare till namnparameterblock.
 *
 *      Ut: Inget.
 *
 *      FV: Returnerar anropade rutiners status.
 *
 *      (C)microform ab 26/12/86 J. Kjellander
 *
 ******************************************************!*/

{
short  nref = 0;                /* Number of referenses */ 
short  i;                       /* loop variable */
GMGRP grprec;                   /* GM struct fr GROUP */
V2REFVA  refarr[ GMMXGP + 1 ];  /* referens array */

/*
***Gruppens namn.
*/
   *( pv[1].par_va.lit.str_va + GMGNLN ) = '\0';
   strcpy( grprec.name_gp, pv[1].par_va.lit.str_va );
/*
***Skapa refarr.
*/
   for ( i = 2; i <= parcount ; i++ )
     {
     refarr[ nref ].seq_val = pv[i].par_va.lit.ref_va[ 0 ].seq_val;
     refarr[ nref ].ord_val = pv[i].par_va.lit.ref_va[ 0 ].ord_val;
     refarr[ nref ].p_nextre = pv[i].par_va.lit.ref_va[ 0 ].p_nextre;
     nref++;
     }
/*
***Frbered grprec fr GM.
*/
   grprec.hed_gp.blank = npblockp->blank;
   grprec.hed_gp.pen = npblockp->pen;
   grprec.hed_gp.level = npblockp->level;
   grprec.nmbr_gp = nref;
/*
***Exekvera GROUP.
*/
   return(EXgrp( ident, refarr,  &grprec));

}

/********************************************************/
