/**********************************************************************
*
*    inac19.c
*    ========
*
*    This file is part of the VARKON Program Module Library.
*    URL: http://www.varkon.com
*
*    short evchr()     evaluate CHR function 
*    short evasci()    evaluate ASCII function 
*    short evstr()     evaluate STR function 
*    short evval()     evaluate VAL function
*    short evleng()    evaluate LENGTH function 
*    short evsubs()    evaluate SUBSTR function 
*    short mkform()    make I/O format string
*
*    This library is free software; you can redistribute it and/or
*    modify it under the terms of the GNU Library General Public
*    License as published by the Free Software Foundation; either
*    version 2 of the License, or (at your option) any later version.
*
*    This library is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*    Library General Public License for more details.
*
*    You should have received a copy of the GNU Library General Public
*    License along with this library; if not, write to the Free
*    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
*    (C)Microform AB 1984-1999, Johan Kjellander, johan@microform.se
*
***********************************************************************/

#include "../../DB/include/DB.h"
#include "../../IG/include/IG.h"
#include "../include/indef.h"

/*
***standard types
*/
extern pm_ptr  stintp;
extern pm_ptr  stflop;
extern pm_ptr  stvecp;
extern pm_ptr  strefp;
extern pm_ptr  ststrp;

/********************************************************/
/*!******************************************************/

        short evchr(
        PMPARVA  pv[],
        PMLITVA *valp)

/*      Evaluate function CHR.
 *
 *      In:    pv[]  =>  Prameter value array 
 *
 *      Out:  *valp  =>  C-pointer to result value
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-12 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
   register v2int code;

   code = pv[1].par_va.lit.int_va;

/*
***check value for code , >= 0 and < 256 , 880128-JK
*/
   if ( ( code < 0 ) || ( code > 255 ) )
       {  
       return( erpush( "IN2092", "CHR" ) );  /* Ilegal value on partameter to CHR */
       }
/*
***execute CHR , result type STRING
*/
   valp->lit.str_va[0] = (char)code;
   valp->lit.str_va[1] = '\0';

   return( 0 );
  }  

/********************************************************/
/*!******************************************************/

        short evasci(
        PMPARVA  pv[],
        PMLITVA *valp)

/*      Evaluate function ASCII.
 *
 *      In:    pv[]  =>  Prameter value array 
 *
 *      Out:  *valp  =>  C-pointer to result value
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-12 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
/*
***execute ASCII , result type INT
*/
/*
***unsigned char 1999-03-05, JK
*/
   valp->lit.int_va = (v2int)(unsigned char)pv[1].par_va.lit.str_va[0];

   return( 0 );
  }

/********************************************************/
/*!******************************************************/

        short evstr(
        PMPARVA  pv[],
        PMLITVA *valp)

/*      Evaluate function STR.
 *
 *      In:    pv[]  =>  Prameter value array 
 *
 *      Out:  *valp  =>  C-pointer to result value
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-12 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
   int format1;                 /*       - " -          */
   int format2;                 /*       - " -          */
   char formstr[ IOFSTRL ];     /* the c:s format string */

   format1 = (int)pv[ 2 ].par_va.lit.int_va;
   format2 = (int)pv[ 3 ].par_va.lit.int_va;
/*
***make c:s format string
*/
   mkform( C_FLO_VA, format1, format2, formstr );
/*
***execute STR, result type STRING
*/
   sprintf( valp->lit.str_va, formstr, pv[1].par_va.lit.float_va );

   return( 0 );
  }

/********************************************************/
/*!******************************************************/

        short evval(
        PMPARVA  pv[],
        PMLITVA *valp)

/*      Evaluate function VAL.
 *
 *      In:    pv[]  =>  Prameter value array 
 *
 *      Out:  *valp  =>  C-pointer to result value
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-12 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
/*
*** execute VAL , result type FLOAT 
*/
   if ( sscanf(pv[1].par_va.lit.str_va,"%lf",&valp->lit.float_va) == 1 )
     {
     return(0);
     }
   else
     {
     valp->lit.float_va = 0.0;
     return(0);
    }
  }

/********************************************************/
/*!******************************************************/

        short evleng(
        PMPARVA  pv[],
        PMLITVA *valp)

/*      Evaluate function LENGTH.
 *
 *      In:    pv[]  =>  Prameter value array 
 *
 *      Out:  *valp  =>  C-pointer to result value
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-12 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
/*
***execute LENGTH , result type INT
*/
   valp->lit.int_va = strlen( pv[ 1 ].par_va.lit.str_va );

   return( 0 );
  }

/********************************************************/
/*!******************************************************/

        short evsubs(
        PMPARVA  pv[],
        PMLITVA *valp)

/*      Evaluate function SUBSTR.
 *
 *      In:    pv[]  =>  Prameter value array 
 *
 *      Out:  *valp  =>  C-pointer to result value
 *
 *      FV:   return - error severity code
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-12 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
   register int   nr_char;
   register char *dest,*source;
   register int   sp,sl,nr_poss;

/*
***Se till att startposition >= 1.
*/
   if ( (sp=pv[2].par_va.lit.int_va) < 1 ) sp = 1;
/*
***Kolla att startposition ligger inom strngen.
*/
   if ( sp > (sl=strlen(pv[1].par_va.lit.str_va)) )
     {
     valp->lit.str_va[0] = '\0';
     return(0);
     }
/*
***Berkna antal tecken att kopiera.
*/
   nr_char = pv[3].par_va.lit.int_va;
   nr_poss = sl - sp + 1;

   if ( nr_char > nr_poss || nr_char < 1 ) nr_char = nr_poss;
/*
***Kopiera nr_char tecken.
*/
   dest = valp->lit.str_va;
   source = &pv[1].par_va.lit.str_va[sp-1];

   for ( ; nr_char>0; ++dest,++source,--nr_char ) *dest = *source;
/*
***Null sist.
*/
   *dest = '\0';

   return(0);
  }  

/********************************************************/
/*!******************************************************/

        short mkform(
        pmvaty  typ,     
        int     form1,   
        int     form2,   
        char   *formstr)

/*      Make I/O format string.
 *
 *      In:    typ    =>  I/O data type 
 *             form1  =>  I/O format1 value 0 indicates default
 *             form2  =>  I/O format2        - " -             
 *
 *      Out: *formstr =>  C:s I/O format string 
 *
 *      (C)microform ab 1985-10-23 Per-Ove Agne'
 *
 *      1999-11-12 Rewritten, R. Svedin
 *
 ******************************************************!*/

  {
   switch ( typ )
     {
     case C_INT_VA:

/*
***no format and input
*/
         if ( form1 == 0 )
             strcpy(formstr,"%d");
         else
             sprintf( formstr, "%%%dd",form1 );
     break;

     case C_FLO_VA:
/*
***no format and input
*/
         if ( form1 == 0 ) 
             {
             strcpy(formstr,"%lf");
             }
         else if ( form2 < 0 )
             {
/*
***output and exponent notation
*/
             if ( form2 == -1 )
                 { 
                 form2 = form1 - 7;
                 if ( form2 > 13 )
                     form2 = 13;

                 if ( form2 >= 0 )
                     sprintf( formstr, "%%%d.%dle", form1, form2 );
                 else
                     sprintf( formstr, "%%%dle", form1 );
                 }
             else
/*
***input
*/
                 {
                 sprintf( formstr, "%%%dlf", form1 );
                 }
             }
         else
             {
             sprintf( formstr, "%%%d.%dlf", form1, form2 );
             }
     break;

     case C_STR_VA:
      
         if ( form2 == -1 )
/*
***output
*/
             { 
             if ( form1 == 0 )
                 strcpy(formstr,"%s");
             else
                 sprintf( formstr, "%%%ds", form1 );
             }
         else
/*
***input
*/
             { 
             if ( form1 == 0 )
                 strcpy(formstr,"%[\136\n]");
             else
                 sprintf( formstr, "%%%d[\136\n]", form1 );
             }

     break;

     }
   return( 0 );
  }

/********************************************************/
