/**********************************************************************
*
*    infunc.c
*    ========
*
*    This file is part of the VARKON Program Module Library.
*    URL: http://www.varkon.com
*
*      inevfu();     Interpreterar funktion
*
*    This library is free software; you can redistribute it and/or
*    modify it under the terms of the GNU Library General Public
*    License as published by the Free Software Foundation; either
*    version 2 of the License, or (at your option) any later version.
*
*    This library is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
*    Library General Public License for more details.
*
*    You should have received a copy of the GNU Library General Public
*    License along with this library; if not, write to the Free
*    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*
*    (C)Microform AB 1984-1999, Johan Kjellander, johan@microform.se
*
***********************************************************************/

#include "../../DB/include/DB.h"
#include "../../IG/include/IG.h"
#include "../include/mbsrout.h"
#include <math.h>

/********************************************************/
/*!******************************************************/

        short inevfu(
        pm_ptr routla,
        pm_ptr arglist,
        PMLITVA *valp,
        pm_ptr *tylap)

/*      Central rutin fr interpretering av funktioner.
 *
 *      In: routla  => Pekare till rutin.
 *          arglist => Pekare till aktuell parameterlista.
 *          valp    => Pekare till resultat.
 *          tylap   => Pekare till resultattyp.
 *
 *      Ut: *valp   => Resultat.
 *          *tylap  => Typ av resultat.
 *
 *      FV: Returnerar ex-rutinens status.
 *
 *      (C)microform ab 23/3/86 J. Kjellander 
 *
 ******************************************************!*/

 {
   STPROC rout;        /* Access structure for MBS routines */
   PMPARVA *pv;        /* pv array for parameter values */
   short parcount;     /* number of actual parameters */
   double p;           /* dummy argument for c-function modf() */
   short status;

/*
***Ls rutin-info frn symboltabellen.
*/
   if ( ( status = strrou( routla, &rout ) ) < -1 )
      return( status );
/*
***Evaluera funktionens parameterlista.
*/
   if ( ( status = inapar( arglist, routla, &rout, &pv, &parcount ) ) != 0 )
      {
      inrpar();
      return(status);
      }
/*
***Resultat-typ.
*/
   valp->lit_type = pv[0].par_va.lit_type;
   *tylap = pv[0].par_ty;
/*
***Exekvera funktionen.
*/
   status = 0;
   switch ( rout.kind_pr )
     {
/*
***Matematiska funktioner.
*/
     case VACOS:
     valp->lit.float_va = RDTODG*(v2float)acos(pv[1].par_va.lit.float_va);
     break;

     case VASIN:
     valp->lit.float_va = RDTODG*(v2float)asin( pv[1].par_va.lit.float_va);
     break;

     case VATAN:
     valp->lit.float_va = RDTODG*(v2float)atan( pv[1].par_va.lit.float_va);
     break;

     case VCOS:
     valp->lit.float_va = (v2float)cos(pv[1].par_va.lit.float_va*DGTORD);
     break;

     case VSIN:
     valp->lit.float_va = (v2float)sin(pv[1].par_va.lit.float_va*DGTORD);
     break;

     case VTAN:
     valp->lit.float_va = (v2float)tan(pv[1].par_va.lit.float_va*DGTORD);
     break;

     case VLN:
     valp->lit.float_va = (v2float)log( pv[1].par_va.lit.float_va );
     break;

     case VLOG:
     valp->lit.float_va = (v2float)log10( pv[1].par_va.lit.float_va );
     break;

     case VSQRT:
     valp->lit.float_va = (v2float)sqrt( pv[1].par_va.lit.float_va );
     break;

     case VABS:
     status = evabs( arglist, valp, tylap );
     break;

     case VFRAC:
     valp->lit.float_va = (v2float)modf( pv[1].par_va.lit.float_va, &p );
     break;

     case VROUND:
     if ( pv[1].par_va.lit.float_va < 0.0 )
          valp->lit.int_va = (v2int)( pv[1].par_va.lit.float_va - 0.5 );
     else
          valp->lit.int_va = (v2int)( pv[1].par_va.lit.float_va + 0.5 );
     break;

     case VTRUNC:
     valp->lit.int_va = (v2int) pv[1].par_va.lit.float_va ;
     break;

#ifndef ANALYZER

     case VRAND:
     status = evrand(pv,valp,parcount);
     break;

     case VREFC:
     status = evrefc( pv, valp, parcount );
     break;

     case VGLOREF:
     status = evglor( pv, valp );
     break;

#endif

/*
***Geometriska funktioner.
*/
     case VVEC:
     status = evvec( pv, valp );
     break;

     case VVECP:
     status = evvecp( pv, valp );
     break;

#ifndef ANALYZER

     case VAREA:
     status = evarea( pv, valp, parcount );
     break;

     case VSUAREA:
     status = evsuar( pv, valp );
     break;

     case VSEAREA:
     status = evsear( pv, valp );
     break;

     case VSECGRAV:
     status = evsecg( pv, valp );
     break;

     case VCGRAV:
     status = evcgra( pv, valp, parcount );
     break;

     case VINTERS:
     status = evsect( pv, valp );
     break;

     case VNINTER:
     status = evnsec( pv, valp );
     break;

     case VARCL:
     status = evarcl( pv, valp );
     break;

     case VIARCL:
     status = eviarc( pv, valp );
     break;

     case VTEXTL:
     status = evtxtl( pv, valp );
     break;

     case VTANG:
     status = evtang( pv, valp );
     break;

     case VITANG:
     status = evitan( pv, valp );
     break;

     case VCURV:
     status = evcurv( pv, valp );
     break;

     case VICURV:
     status = evicur( pv, valp );
     break;

     case VCENTRE:
     status = evcen( pv, valp );
     break;

     case VNORM:
     status = evnorm( pv, valp, parcount );
     break;

     case VEND:
     status = evend( pv, valp );
     break;

     case VSTART:
     status = evstrt( pv, valp );
     break;

     case VON:
     status = evon( pv, valp, parcount );
     break;

     case VION:
     status = evion( pv, valp );
     break;

     case VIONSUR:
     status = evions( pv, valp, parcount );
     break;

     case VEVAL:
     status = eveval( pv, valp );
     break;

     case VNCURI:
     status = evncui( pv, valp );
     break;

     case VIDENT:
     status = evidnt( pv, valp, parcount);
     break;

     case VGTPTR:
     status = evgptr(pv, valp);
      break;

     case VPOS:
     status = evpos( pv, valp, parcount);
      break;

     case VSCREEN:
     status = evscr( pv, valp, parcount);
     break;

     case VGNXGM:
     status = evgngm( pv, valp);
     break;

     case VGTID:
     status = evgtid( pv, valp );
     break;

     case VGETWDT:
     status = evgwdt( pv, valp );
     break;

     case VGTTYP:
     status = evgtyp( pv, valp );
     break;

     case VGTINT:
     status = evgint( pv, valp );
     break;

     case VGTFLT:
     status = evgflt( pv, valp );
     break;

     case VGTSTR:
     status = evgstr( pv, valp );
     break;

     case VGTVEC:
     status = evgvec( pv, valp );
     break;

     case VGTREF:
     status = evgref( pv, valp );
     break;

     case VVECL:
     status = evvecl( pv, valp );
     break;

     case VVECN:
     status = evvecn( pv, valp );
     break;

     case VANGLE:
     status = evangl( pv, valp );
     break;

     case VVPROD:
     status = evcrpr( pv, valp );
     break;
/*
***MBS-funktioner.
*/
     case VADDMBS:
     status = evaddm(pv,valp,parcount);
     break;

     case VGPMBS:
     status = evgpm( pv, valp );
     break;

     case VPOSMBS:
     status = evposm( pv, valp );
     break;
/*
***vriga funktioner.
*/
     case VACLEV:
     status = evalev( valp );
     break;

     case VACBLK:
     status = evablk( valp );
     break;

     case VACVIN:
     status = evavin( valp );
     break;

     case VACPEN:
     status = evapen( valp );
     break;

     case VACTWDT:
     status = evawdt( valp );
     break;

     case VACSCL:
     status = evascl( valp );
     break;

     case VACDSCL:
     status = evadsc( valp );
     break;

     case VACCAC:
     status = evacac( valp );
     break;

     case VACGRX:
     status = evagrx( valp );
     break;

     case VACGRY:
     status = evagry( valp );
     break;

     case VACGRDX:
     status = evagdx( valp );
     break;

     case VACGRDY:
     status = evagdy( valp );
     break;

     case VACGRID:
     status = evagrd( valp );
     break;

     case VACJOBN:
     status = evajbn( valp );
     break;

     case VACMTYP:
     status = evamtp( valp );
     break;

     case VACMATR:
     status = evamat( valp );
     break;

     case VACJOBD:
     status = evajbd( valp );
     break;

     case VACLFNT:
     status = evalft( valp );
     break;

     case VACAFNT:
     status = evaaft( valp );
     break;

     case VACCFNT:
     status = evacft( valp );
     break;

     case VACXFNT:
     status = evaxft( valp );
     break;

     case VACLDSL:
     status = evaldl( valp );
     break;

     case VACADSL:
     status = evaadl( valp );
     break;

     case VACCDSL:
     status = evacdl( valp );
     break;

     case VACXDSL:
     status = evaxdl( valp );
     break;

     case VACTSIZ:
     status = evatsi( valp );
     break;

     case VACTWID:
     status = evatwi( valp );
     break;

     case VACTSLT:
     status = evatsl( valp );
     break;

     case VACDTSZ:
     status = evadts( valp );
     break;

     case VACDASZ:
     status = evadas( valp );
     break;

     case VACDNDG:
     status = evadnd( valp );
     break;

     case VACDAUT:
     status = evadau( valp );
     break;

     case VACTCSY:
     status = evacsy( valp );
     break;

     case VAPID:
     status = evapid( valp );
     break;

     case VAVVERS:
     status = evavvr( valp );
     break;

     case VAVSER:
     status = evavsr( valp );
     break;

     case VAOSTYP:
     status = evaost( valp );
     break;

     case VAHOST:
     status = evahst( valp );
     break;

     case VGETENV:
     status = evgenv( pv, valp );
     break;

     case VGTTSTR:
     status = evgtts( pv, valp );
     break;

     case VGTALT:
     status = evgtal( pv, valp );
     break;

     case VEXEST:
     status = evexst(valp);
     break;

     case VLDDLL:
     status = evlddl(pv,valp);
     break;

     case VCRAYNPES:
     status = evcrnp(valp);
     break;

     case VCRAYMYPE:
     status = evcrmp(valp);
     break;
/*
***Fnster-funktioner.
*/
     case VCRWIN:
     status = evcrwi( pv, valp );
     break;

     case VCREDI:
     status = evcred( pv, valp );
     break;

     case VCRBUT:
     status = evcrbu( pv, valp, parcount );
     break;

     case VCRFBUT:
     status = evcrfb( pv, valp );
     break;

     case VCRDBUT:
     status = evcrdb( pv, valp, parcount );
     break;

     case VCRICO:
     status = evcric( pv, valp, parcount );
     break;

     case VCRFICO:
     status = evcrfi( pv, valp );
     break;

     case VGTBUT:
     status = evgtbu( pv, valp );
     break;

     case VGTEDI:
     status = evgted( pv, valp );
     break;

     case VWTWIN:
     status = evwtwi( pv, valp );
     break;

     case VTXTLW:
     status = evgttl( pv, valp, parcount );
     break;

     case VTXTHW:
     status = evgtth( pv, valp, parcount );
     break;
/*
***I/U-funktioner.
*/
     case VININT:
     status = evinio( C_INT_VA, pv, valp );
     break;

     case VINFLT:
     status = evinio( C_FLO_VA, pv, valp );
     break;

     case VINSTR:
     status = evinio( C_STR_VA, pv, valp );
     break;

     case VINLIN:
     status = evinl( pv, valp );
     break;

     case VFPOS:
     status = evfpos( pv, valp );
     break;

     case VIOSTAT:
     status = eviost( pv, valp );
     break;

     case VUFNAM:
     status = evufnm(valp);
     break;

     case VINPMT:
     status = evinpt( pv, valp );
     break;

     case VINFNM:
     status = evinfn( pv, valp );
     break;

     case VTSTFIL:
     status = evtstf( pv, valp );
     break;

#endif
/*
***Strnghantering.
*/
     case VCHR:
     status = evchr( pv, valp );
     break;

     case VASCII:
     status = evasci( pv, valp );
     break;

     case VSTR:
     status = evstr( pv, valp );
     break;

     case VVAL:
     status = evval( pv, valp );
     break;

     case VLENGTH:
     status = evleng( pv, valp );
     break;

     case VSUBSTR:
     status = evsubs( pv, valp );
     break;

#ifndef ANALYZER

     case VRSTR:
     status = evrstr( pv, valp );
     break;

     case VRVAL:
     status = evrval( pv, valp );
     break;

     case VIVAL:
     status = evival( pv, valp, parcount);
     break;

     case VFVAL:
     status = evfval( pv, valp, parcount);
     break;

     case VFINDS:
     status = evfins( pv, valp );
     break;
/*
***Databasrutiner fr RDB.
*/
     case VDBOP:
     status = evdbop( pv, valp );
     break;

     case VDBBG:
     status = evdbbg( pv, valp );
     break;

     case VDBSL:
     status = evdbsl( pv, valp, parcount );
     break;

     case VDBUP:
     status = evdbup( pv, valp, parcount );
     break;

     case VDBIN:
     status = evdbin( pv, valp, parcount );
     break;

     case VDBDL:
     status = evdbdl( pv, valp );
     break;

     case VDBEN:
     status = evdben( valp );
     break;

     case VDBRL:
     status = evdbrl( valp );
     break;

     case VDBCL:
     status = evdbcl( valp );
     break;

     case VDBFI:
     status = evdbfi( pv, valp );
     break;

     case VDBFF:
     status = evdbff( pv, valp );
     break;

     case VDBFS:
     status = evdbfs( pv, valp );
     break;

     case VDBNX:
     status = evdbnx( valp );
     break;
/*
***Databasrutiner fr ODBC.
*/
     case VODBCCO:
     status = evodco( pv, valp );
     break;

     case VODBCED:
     status = evoded( pv, valp );
     break;

     case VODBCFE:
     status = evodfe( valp );
     break;

     case VODBCGS:
     status = evodgs( pv, valp );
     break;

     case VODBCDC:
     status = evoddc( valp );
     break;

     case VODBCER:
     status = evoder( valp );
     break;
/*
***Databasrutiner fr GM.
*/
     case VPUTDAT:
     status = evpdat( pv, valp );
     break;

     case VGETDAT:
     status = evgdat( pv, valp );
     break;

     case VDELDAT:
     status = evddat( pv, valp );
     break;

#endif

     default:
     inrpar();
     return( erpush( "IN0104" , "" ) );
     break;
     }
/*
***Frigr parameter-pool utrymme.
*/
   inrpar();                     /* release parameter pool space */
/*
***Felkontroll.
*/
   if ( status < -1 ) return(status);
   else return(0);
 }

/************************************************************************/
